;==========================================================================================================================
;       18F452 - 40 Mhz - Crystal 10 Mz run with HSPLL activated - Programmateur de chauffage
;       Auteur : Michel Durrenbach
;==========================================================================================================================

	list      p=18F452             ; list directive to define processor
	#include <p18F452.inc>         ; processor specific variable definitions
;     Configuration Registers

	__CONFIG    _CONFIG1H, _OSCS_OFF_1H & _HSPLL_OSC_1H
	__CONFIG    _CONFIG2L, _BOR_OFF_2L & _PWRT_ON_2L
	__CONFIG    _CONFIG2H, _WDTPS_16_2H 				; WatchDog Timer On X 16 = 288 ms
	__CONFIG    _CONFIG3H, _CCP2MX_OFF_3H
	__CONFIG    _CONFIG4L, _STVR_OFF_4L & _LVP_OFF_4L & _DEBUG_OFF_4L
	__CONFIG    _CONFIG5L, _CP0_OFF_5L & _CP1_OFF_5L & _CP2_OFF_5L & _CP3_OFF_5L
	__CONFIG    _CONFIG5H, _CPB_OFF_5H & _CPD_OFF_5H
	__CONFIG    _CONFIG6L, _WRT0_OFF_6L & _WRT1_OFF_6L & _WRT2_OFF_6L & _WRT3_OFF_6L
	__CONFIG    _CONFIG6H, _WRTC_OFF_6H & _WRTB_OFF_6H & _WRTD_OFF_6H
	__CONFIG    _CONFIG7L, _EBTR0_OFF_7L & _EBTR1_OFF_7L & _EBTR2_OFF_7L & _EBTR3_OFF_7L
	__CONFIG    _CONFIG7H, _EBTRB_OFF_7H

;==========================================================================================================================
; The GPS cable is connected to RC7 (serial RS232 input). It receives 200 sentences each minute at 4800 bits/sec
; The sentences used are RMC and GGA only, receiving each once by second.
; This program use High priority interrupt to receive all data and to sort NMEA sentences.
; The main program is interrupted each 2 ms to receive NMEA data and also each 1 ms by timer 3 to sort these data
; Time is given in Greenwich time, so the program takes care of summertime and wintertime to add + 1 or + 2 hours
; To do this, tables are used with the 28 years cycle found, a March and an October table
;==========================================================================================================================
;  type   time         latitude    longitude                 date    CRC
; $GPRMC,093300.000,A,4721.4514,N,00221.9622,W,19.12,346.71,140406,,*11
; $GPGGA,093301.000,4721.4514,N,00221.9625,W,1,04,4.8,33.9,M,49.2,M,,0000*75
; The fields are separated by "," the field lenght is variable ! so to find data the program count the number of ","
;==========================================================================================================================
; PortA |   Pin  | I/O | Name          |
;       |        |     |               |
;  RA0  |     2  |  I  | 		   |
;  RA1  |     3  |  I  | 		   |
;  RA2  |     4  |  I  | 		   |
;  RA3  |     5  |  I  |               |
;  RA4  |     6  |  I  |  		   |
;  RA5  |     7  |  I  |               |
;==========================================================================================================================
; PortB |   Pin  | I/O | Name          | Port B internal pullup activated
;       |        |     |               |
;  RB0  |    33  |  I  | 		   | Key + temperature NO (active when closed)
;  RB1  |    34  |  I  | 		   | Key - temperature NO (active when closed)
;  RB2  |    35  |  I  | 		   |
;  RB3  |    36  |  I  |               |
;  RB4  |    37  |  I  |  		   |
;  RB5  |    38  |  I  |               |
;  RB6  |    39  |  I  | 		   | Log switch NO (active when closed)
;  RB7  |    40  |  I  | 		   | Grounded for Machine # 2, opened for machine # 1
;==========================================================================================================================
; PortC |   Pin  | I/O | Name          | PORT C description
;       |        |     |               |
;  RC0  |    15  |  I  |               |
;  RC1  |    16  |  I  |  		   |
;  RC2  |    17  |  I  |  		   |
;  RC3  |    18  |  O  | SCL		   | Serial clock for I2C bus connected to DS75 and P8583 real time clock 3.6 K pullup
;  RC4  |    23  | I/O | SDA 		   | Serial data  for I2C bus connected to DS75 and P8583 real time clock 3.6 K pullup
;  RC5  |    24  |  I  |  		   |
;  RC6  |    25  |  I  | 		   |
;  RC7  |    26  |  I  | Rdata 	   | Receive data RS232 (GPS) at 4800 bits/sec
;==========================================================================================================================
; PortD |   Pin  | I/O | Name          | Port D description
;       |        |     |               | Port D Lcd Command
;  RD0  |    19  |  O  | RS		   | Lcd RS
;  RD1  |    20  |  O  | R/W		   | Lcd R/W
;  RD2  |    21  |  O  | E1		   | Lcd 1 Enable first display  (2 lines)
;  RD3  |    22  |  O  | E2            | Lcd 2 Enable second display (2 lines)
;  RD4  |    27  |  O  | DB4 		   | Lcd bit 4
;  RD5  |    28  |  O  | DB5           | Lcd bit 5
;  RD6  |    29  |  O  | DB6  	   | Lcd bit 6
;  RD7  |    30  |  O  | DB7		   | Lcd bit 7
;==========================================================================================================================
; PortE |   Pin  | I/O | Name          | Description
;       |        |     |               | Port E 3 pins in OUTPUT
;  RE0  |    8   |  O  | Reed relay	   | Activate Reed relay
;  RE1  |    9   |  O  | Led Power ON  | Activate Power Led
;  RE2  |   10   |  O  | 		   |
;========== Variables =====================================================================================================

	cblock		0x0000
	Work4			; Work REGISTER
	Work3			; Work REGISTER
	Work2			; Work REGISTER
	Work1			; Work REGISTER
	Work0			; Work REGISTER
	Work8			; Work REGISTER
	Work7			; Work REGISTER
	Work6			; Work REGISTER
	Work5			; Work REGISTER

	W1			; Work register used with high priority interrupt
	W3			; Work register used with high priority interrupt
	W5			; Work register used with high priority interrupt
	W6			; Work register used with high priority interrupt

	dec1
	dec2
	hex1
	Day
	Month
	Year
	Heure
	Minute
	Seconde

	FLAG		      ; Various usage
	FLAG_
	NMEA			;
	NMEA2
	Byte_count		;

	Degre_Hex_H		;
	Degre_Hex_L		;

	Degre_Dec_H		;
	Degre_Dec_L		;

	LCD_TEMP	      ; Variable for LCD
	LCD_CHAR	      ; Variable for LCD
	LCD_CHAR_HEXA	; Variable for LCD
	ptr_count	      ;
	ptr_pos		;
	I2C_ADDR	      ;
	I2C_DATA	      ;
	EE_ADDR 	      ; Eeprom Address
	EE_DATA		; Eeprom Data
	GMT 			; GMT should be 1 or 2
	Day_Month		;
	Sat_detected	; Satellites detected
	Degre			; Degre in decimal
	Degre10		; tenth of degre in decimal
	Degre10D
	DegreD
	Store_COUNT_H	; Number of write in EEPROM
	Store_COUNT_L	; Number of write in EEPROM
	POSITIVE		;
	DJU_H			; DJU_H and DJU_L contains the hexa value of the Temperature
	DJU_L			; defined in the ROTACTOR - the TEMP_24_H and TEMP_24_L
	Degre_Rot		; Temperature in Rotactor Degre
	Degre10_Rot		; Temperature in rotactor in 1/10 Degre
	DJU_H_true		; This is the True DJU value corrected with value read at -12 hours
	DJU_L_true		; it is done to extrapole the most predictive temperature
	DegreH		; Degre in Hexa
	Degre10H		; 1/10 Degre in Hexa
	TEMP_24_H		; TEMP_24_H contains the hexa value of cumuled temperature
	TEMP_24_L		; TEMP_24_L on 24 hours updated each 30 mn
	DJU_H2		; programmed temperature minus actual temperature High
	DJU_L2		; programmed temperature minus actual temperature Low
	START_TIME_H	; Start time for Night time Heure
	START_TIME_M	; Start time for Night time Minute
	START_TIME_H2	; Start time for Day time Heure
	START_TIME_M2	; Start time for Day time Minute
	END_TIME_H		; End time for Night time Heure
	END_TIME_M		; End time for Night time Minute
	END_TIME_H2		; End time for Daytime Heure
	END_TIME_M2		; End time for Daytime Minute
	TOT_H			; POWER ON Total duration High
	TOT_L			; POWER ON Total duration Low
	HC_H			; POWER ON en heures creuses High
	HC_L			; POWER ON en heures creuses Low
	HP_H			; POWER ON en heures pleines High
	HP_L			; POWER ON en heures pleines Low
	PriorityH
	PriorityL
	GGA
	RMC
	TOTAL
	MAL			; total of update time (once a day from the last power ON)
	CRC
	MH
	ML
	RU
	RH
	RL
	endc

;========== Constants =====================================================================================================

#define		DS75_ADDR		0x90		  ; adresse DS75 = 10010000    R/W(0)
#define		P8583_ADDR		0xA0		  ; adresse 8583 = 10100000    R/W(0)
#define  		LCD_RS		B'00000001'	  ; LCD RS
#define		LCD_RW		B'00000010'   ; LCD R/W
#define    		LCD_E1		2		  ; LCD 1 Enable
#define    		LCD_E2		3		  ; LCD 2 Enable

LCD_DATA_PORT	EQU			PORTD		  ; LCD port
LCD_DATA_TRIS	EQU			TRISD		  ; LCD config

;==========================================================================================================================
;
;FLAG bit 0 1 2 3 4 5 6 7
;
;     bit 0	; $ received (start of NMEA sentence)
;     bit 1	; NMEA buffer complete
;     bit 2	; select display 1 or 2
;     bit 3	; if set, means it is not 00 nor 30 minutes
;     bit 4	; bad CRC in NMEA sentence
;     bit 5	; * received in NMEA
;     bit 6	;
;     bit 7	; RMC sentence received
;
;FLAG_ bit 0 1 2 3 4 5 6 7
;
;     bit 0	; update time from RMC sentence at 03.00 AM
;     bit 1	; It is time to update P8583 device
;     bit 2	; setup time to EEPROM is done
;     bit 3	;
;     bit 4	;
;     bit 5	; power ON detected Heures creuses
;     bit 6	; power ON detected Heures Pleines
;     bit 7	; key select
;
;========= Special characters loaded into CGRAM ===========================================================================
;
;	0x00 = 
;	0x01 = 
;	0x02 = 
;	0x03 = 
;
;========= MACROS =========================================================================================================

clrc		macro				; clear carry bit
		bcf	 STATUS,C
		endm

setc		macro				; set carry bit
		bsf	 STATUS,C
		endm
clrz		macro				; clear Z bit
		bcf	 STATUS,Z
		endm

setz		macro				; set Z bit
		bsf	 STATUS,Z
		endm

clrdc		macro				; clear DC bit
		bcf	 STATUS,DC
		endm

skpz		macro				; skip next instruction if Z is set
		btfss	 STATUS,Z
		endm

skpnz		macro				; skip next instruction if Z is clear
		btfsc	 STATUS,Z
		endm

skpc		macro				; skip next instruction if carry is set
		btfss	 STATUS,C
		endm

skpnc		macro				; skip next instruction if carry is clear
		btfsc	 STATUS,C
		endm

skpov		macro				; skip next instruction if overflow is set
		btfss	 STATUS,OV
		endm

skpnov	macro				; skip next instruction if overflow is clear
		btfsc	 STATUS,OV
		endm

MOVLF		macro	 literal,dest	; move literal to destination (REG)
		movlw	 literal
		movwf	 dest
		endm

POINT		macro	 stringname
		MOVLF	 upper stringname,TBLPTRU
		MOVLF	 high stringname,TBLPTRH
		MOVLF	 low stringname,TBLPTRL
		endm

CMPLR		macro	 literal,reg			; compare literal against REG
		movlw	 literal
		xorwf	 reg,W
		endm

CLLRR		macro	 literal,literal1,reg,reg1	; compare 2 literals against 2 REG
		movlw	 literal
		xorwf	 reg,W
		bnz	 $+6
		movlw	 literal1
		xorwf	 reg1,W
		endm

CMPRR		macro	 reg1,reg				; compare 2 REG
		movf	 reg,W
		xorwf	 reg1,W
		endm

CRrRr		macro	 reg1,reg2,reg3,reg4		; compare 2 REG's against 2 REG's
		movf	 reg1,W
		xorwf	 reg2,W
		bnz	 $+6
		movf	 reg3,W
		xorwf	 reg4,W
		endm

TSTZ		macro reg1,reg2				; test 2 registers, set Z if both are 0
		setz
		tstfsz reg1
		clrz
		tstfsz reg2
		clrz
		endm

INCREG	macro	 reg					; incremente REG in BCD from 0 to 99
		incf	 reg,W
		daw
		movwf	 reg
		endm

DELAY		macro	 literal				; delay number of ms (IE : DELAY 50 = wait 50 ms)
		movlw	 literal
		call   Delay_
		endm

SPACES	macro	 literal				; display number of spaces specified (ie SPACES 6)
		movlw	 literal
		movwf	 Work8
		call   LCD_Space
		decf   Work8
		bnz	 $-6
		endm

KEY1_2P	macro						; check for 2 keys pressed
		movff	 PORTB,WREG
		andlw	 b'00000011'
		xorlw	 b'00000000'
		endm

KEY1_2R	macro						; check for 2 keys released
		movff	 PORTB,WREG
		andlw	 b'00000011'
		xorlw	 b'00000011'
		endm

LCD1		macro	 literal,reg1			; display 1 REG at a given display address
		movlw	 literal
		call	 LCD_Locate
		movf	 reg1,W
		call	 LCD_Hexa
		endm

LCD2		macro	 literal,reg1,reg2		; display 2 REG at a given display address
		movlw	 literal
		call	 LCD_Locate
		movf	 reg1,W
		call	 LCD_Hexa
		movf	 reg2,W
		call	 LCD_Hexa
		endm

LCD3		macro	 literal,reg1,reg2,reg3		; display 3 REG at a given display address
		movlw	 literal
		call	 LCD_Locate
		movf	 reg1,W
		call	 LCD_Hexa
		movf	 reg2,W
		call	 LCD_Hexa
		movf	 reg3,W
		call	 LCD_Hexa
		endm

LCD4		macro	 literal,reg1,reg2,reg3,reg4	; display 4 REG at a given display address
		movlw	 literal
		call	 LCD_Locate
		movf	 reg1,W
		call	 LCD_Hexa
		movf	 reg2,W
		call	 LCD_Hexa
		movf	 reg3,W
		call	 LCD_Hexa
		movf	 reg4,W
		call	 LCD_Hexa
		endm

WLI2C		macro	literal,reg1	  	;
		MOVLF	literal,reg1
		call	WrDATA		  	; sends data to 8583
		endm

WRI2C		macro	reg,reg1		 	;
		movff	reg,reg1		 	;
		call	WrDATA		  	; sends data to 8583
		endm

WCGRAM	macro lit,lit0,lit1,lit2,lit3,lit4,lit5,lit6,lit7
		movlw	lit
		call	LCD_CGRAM
		movlw	lit0
		call	LCD_Char

		movlw	lit+1
		call	LCD_CGRAM
		movlw	lit1
		call	LCD_Char

		movlw	lit+2
		call	LCD_CGRAM
		movlw	lit2
		call	LCD_Char

		movlw	lit+3
		call	LCD_CGRAM
		movlw	lit3
		call	LCD_Char

		movlw	lit+4
		call	LCD_CGRAM
		movlw	lit4
		call	LCD_Char

		movlw	lit+5
		call	LCD_CGRAM
		movlw	lit5
		call	LCD_Char

		movlw	lit+6
		call	LCD_CGRAM
		movlw	lit6
		call	LCD_Char

		movlw	lit+7
		call	LCD_CGRAM
		movlw	lit7
		call	LCD_Char
		endm

		; this macro multiply a 16 bits value by a 8 bits value result in RU RH and RL
MUL16		macro reg1,reg2,lit3 		; ie: 0x0334 X 0x5A or 820 X 90 = 0x12048
		clrf	RU				; clear result
		clrf	RH				; clear result
		clrf	RL				; clear result
		movff	reg1,MH			; move 0x03 to MH
		movff	reg2,ML			; move 0x34 to ML
		movlw	lit3				; move 0x5A to WREG
		mulwf	ML				; multiply 0x34 by 0x5A = 0x1248
		call	Add_data			; add data to result
							; now multiply by 0x100 (256 decimal)
		movlw	lit3				; move 0x5A to WREG
		mullw	0xFF				; multiply by 0xFF = 0x59A6
		addwf	PRODL				; add once more    = 0x5A00 to make 0x100
		skpnc					; as 0xFF + 1 = 0x100
		incf	PRODH				; adjust result if carry
		movff	MH,Work4			; move 0x03 (high value) to counter
		movf	Work4,W
		bz	$+0x0A 			; test if zero
		call	Add_data			; add 0x5A00 the number of times
		decf	Work4				; decremente original value of 0x03
		bnz	$-6				; do Add_data until zero
		endm					; result is 0x12048

;==========================================================================================================================

		org    0x0000 			; main program
		bra	 main
		org	 0x0008			; High priority interrupt vector
		bra	 RS232_TMR

;========== High priority interrupt =======================================================================================

RS232_TMR						; An interrupt occurs, select the one active
							; Timer 3 interrupts each 1 ms

     IF_ PIR2,TMR3IF == 1			; test if timer 3 interrupt overflow is ON
		bcf	 PIR2,TMR3IF		; reset timer overflow
		bcf	 T3CON,TMR3ON		; Set timer OFF
		MOVLF	 0xD8,TMR3H			; move W to TMR3H 0xD8F0 neg = 0x2710 = 10000 X 100 ns = 1 ms
		MOVLF	 0xF0,TMR3L			; move W to TMR3L
		MOVLF	 B'00000000',T3CON	; Set divide by 1
		call   Look_NMEA			; look at NMEA sentences each 1 ms
		bsf	 T3CON,TMR3ON		; Set timer ON
     ENDIF_

							; receive from GPS interrupts each 2.08 ms at 4800/s (1000/480 bytes/s = 2.08 ms)
     IF_ PIR1,RCIF == 1				; test if UART receiver interrupt flag is ON

	  IF_ FLAG,5 == 0				; Test if * received
		MOVLF	 0x02,Byte_count 		; Set up byte count for receiving 2 CRC
		bcf	 PIR1,RCIF			; Clear UART receiver interrupt flag
		movff	 RCREG,NMEA			; Store data byte from UART to NMEA
		CMPLR	 "$",NMEA			; Compare NMEA with $
		skpnz					; Skip if not $
		bsf	 FLAG,0			; Set FLAG 0 ON, $ received
		btfsc	 FLAG,0			; Test FLAG 0, skip if OFF
		movff  NMEA,POSTINC0		; As FLAG 0 ON, store NMEA byte with auto-increment
		CMPLR	 "*",NMEA			; Compare NMEA with *
		skpnz					; Skip if not *
		bsf	 FLAG,5			; Set FLAG 5 ON, * received
	  ELSE_					; else "*" was received, so it's is a CRC byte
		bcf	 PIR1,RCIF			; Clear UART receiver interrupt flag
		movff  RCREG,POSTINC0		; Store byte with auto-increment
		decf	 Byte_count			; decremente byte count (2 CRC's bytes)
	  ENDIF_

		movf	 Byte_count,W		; prepare to check if STATUS,Z is set

	  IF_ .Z.					; test 2 CRC received, byte count is 0
		bsf	 FLAG,1			; Set FLAG 1 ON, NMEA buffer complete
		lfsr 	 FSR0,0x100			; Set buffer
		bcf	 FLAG,5			; Clear FLAG * received
		incf	 TOTAL
	  ENDIF_

     ENDIF_

		retfie FAST				; return and recover registers

;==========================================================================================================================
;	main program
;==========================================================================================================================

main

		  call Set_UP

  LOOP_

	    IF_ FLAG_,1 == 1			; it is 02.01 AM or 03.01 AM, ready to update P8583 time

		IF_ FLAG,7 == 1			; be sure it is a RMC sentence
		  call  Get_date
		  bcf	  FLAG_,1
		  bsf	  FLAG_,0			; set flag to say time is updated, will be reset at 5.00 AM
		  incf  MAL				; incremente number of times clock update (once a day)
		ENDIF_

	    ENDIF_

	    	  clrwdt				; clear watchdog timer
		  incf  PriorityL			; Count up to 65535
		  skpnz
		  incf  PriorityH

		  TSTZ  PriorityH,PriorityL	; Test both registers for zero
    IF_ .Z.						; Enter this routine once out of 65535, reg's are zero (about each 120 ms)
		  call  P8583_Time		; sends Start, address A1, gets Ack, read data, sends Ack
		  call  Test_00_30
		  call  Get_Temperature_dec	; Get Temperature in Decimal
		  call  Check_EE			; Check if EEPROM contains Temperature, if FF initialize with Degre
		  call  Get_thermo		; look at push buttons if any is pressed
		  call  Compute_DJU		; Compute DJU

	    IF_ FLAG,3 == 1			; FLAG,3 active, so test more
		  call  Degre_to_EE		; Bit 3 active, so store data to EEPROM
		  call  Compute_DJU		; recompute DJU take care of last temperature stored
		  call  DJU_to_EE			; Store new DJU to EEPROM
	    ENDIF_

		  bsf	  FLAG,2			; Set ON bit 2 (display N2)
		  call  Display_Time		; display time and date
		  call  Ete_Hiver			; display 'Heure t' or 'Heure hiver'
		  call  Display_Sat		; display number of satellites
		  call  Display2temp		; display real temperature and thermostat temperature

	  IF_   PORTB,RB6 == 0			; Log button pressed
		  call  Display_log
	  ELSE_

		  CLLRR 0x05,0xA0,TOT_H,TOT_L ; Check for total DURATION = 0x5A0 (1440 mn = 24 hours)

	    IF_ .Z.
		     bsf   LATE,RE0		; POWER ON if TOT_DURATION is 0x5A0 (1440 mn = 24 hours)
		     bsf   LATE,RE1		; Led ON

	    	     bcf   FLAG,2			; Set OFF bit 2 (display N1)
	 	     call  LCD_Clear		; clear display
		     MOVLF d'160',ptr_pos	; CHAUFFAGE 24h/24
		     call  Lcd_char_1
		     MOVLF d'200',ptr_pos
		     call  Lcd_char_2
	    ELSE_
	    	     bcf   FLAG,2			; Set OFF bit 2 (display N1)
		     call  Get_Start_Time_HC	; get start time for heures creuses
		     call  Get_Start_Time_HP	; get start time for heures pleines
		     TSTZ  HP_L,HP_H		; test 2 registers, set Z if both are 0

		   IF_ .Z.
	    	     bcf   FLAG,2			; Set OFF bit 2 (display N1)
		     MOVLF d'80',ptr_pos	; "Pas de chauffage la journe"
		     call  Lcd_char_2
		   ELSE_
		     call  Day_time		; display time "Heures pleines : XXhXX  XXhXXmn
		   ENDIF_

		      TSTZ  HC_L,HC_H		; test 2 registers, set Z if both are 0

		   IF_ .Z.
	    	      bcf   FLAG,2		; Set OFF bit 2 (display N1)
		      MOVLF d'120',ptr_pos	; "Pas de chauffage la nuit"
		      call  Lcd_char_1
		   ELSE_
		      call Night			; display time "Heures creuses : XXhXX  XXhXXmn
		   ENDIF_

          ENDIF_

	  ENDIF_

		movlw	 0x24				; set % to end of first line
		call	 LCD_Locate
		MOVLF	 0xF4,EE_ADDR		; read percentage from EEPROM
		call	 Rd_from_EEPROM
		movwf	 hex1				; move value to hex1
		call	 Hex_Dec			; convert value to decimal back with dec value in dec1 and dec2
		movf	 dec1,W

        IF_ .NZ.
		MOVLF	 1,LCD_CHAR_HEXA
		call	 LCD_Single
	  ELSE_
		SPACES 1
        ENDIF_

		movf	 dec2,W
		call   LCD_Hexa
		movlw	 "%"
		call	 LCD_Char

    ENDIF_

  ENDLOOP_

;========== Display Wintertime or Summertime ==============================================================================

Ete_Hiver

		movlw 0x1D
		call  LCD_Locate

	  IF_ GMT,0 == 1
		POINT Hiver
	  ELSE_
		POINT Ete
	  ENDIF_

		call  LCD_variable
		return

;========== Get month data ================================================================================================

Month_data

		movff	Month,dec2			; move Month into dec2
		call	Dec_Hex			; convert it in Hex
		decf	hex1				; dec by 1 to address the table from 0
		movlw	0x0A
		mulwf	hex1				; multiply by 10 (lenght of each entry) result in PRODH and PRODL
		POINT	MonthTable			; point to the table
		movf	PRODL,W
		addwf	TBLPTRL			; add the offset to table pointer
		skpnc
		incf	TBLPTRH			; adjuts HIGH pointer as required
		return

;========== Look at NMEA received each 1 ms ===============================================================================

Look_NMEA		; process while interrupted

	 IF_ FLAG,1 == 1				 ; test receive complete HIGH PRIORITY ROUTINE
		bcf	 FLAG,0			 ; clear FLAG wait for $
		bcf	 FLAG,1			 ; clear FLAG wait for *
		call	 Check_CRC

	   IF_ FLAG,4 == 0			 ; test if good CRC in NMEA phrase
	      call Sort_NMEA_data		 ; Sort NMEA sentences keep only RMC and GGA
         ENDIF_

	 ENDIF_

		return

;============ Set_UP ======================================================================================================

Set_UP
		lfsr 	 FSR0,0x000			; clear memory from 0x00 to 0x7F
		clrf 	 POSTINC0 			; clear INDF register and inc pointer
		btfss  FSR0L,7			; test bit 7, when set means 0x80 reached, so skip
		bra	 $-4				; no, clear
		MOVLF	 0xFE,CRC			; initialize CRC

		bsf	 FLAG,3			; Set ON bit 3 to say it is not 00 nor 30 mn

		DELAY	 d'50'

		call	 InitIO			; initialize Ports I/O

		bcf	 FLAG,2			; Set OFF bit 2 (display N1)
		call	 LCD_Init
		bsf	 FLAG,2			; Set ON bit 2  (display N2)
		call	 LCD_Init
		call	 I2C_Init

		call	 DS75_Init

		DELAY	 d'50'			; 50 ms setup time for LCD

	  	bcf	 FLAG,2			; Set OFF bit 2 (display N1)
		MOVLF	 d'0',ptr_pos		; Display   "Recherche de"
		call	 Lcd_char_1			;           " Satellites"
		MOVLF	 d'40',ptr_pos
		call	 Lcd_char_2

		call	 Wait_3_Satellites	; runs 3 times to be sure time and date are valid
		call	 Wait_3_Satellites
		call	 Wait_3_Satellites

		lfsr 	 FSR0 ,0x100		; initialize FSR0

	 	call	 LCD_Clear			; clear display
		call	 Get_date

		MOVLF	 0xF0,EE_ADDR		; Set EEPROM address to 0xF0
		call	 Rd_from_EEPROM

		CMPLR	 0xFF,EEDATA		; compare EE_DATA with FF (EEPROM not initialised)

	  IF_ .Z.					; initialize temperature and end of time 'Heures creuses'
		MOVLF	 0xF0,EE_ADDR		; write at 0xF0 18.00 as default temperature (seen as 20C)
		MOVLF	 0x18,EE_DATA		; if not already done (enter this code the first POWER ON only)
		call	 Wr_to_EEPROM		; as once EEPROM is initialized can't never go here
		clrf	 EE_DATA
		call	 Wr_to_EEPROM
		MOVLF	 0x07,EE_DATA		; if not already done write end of 'heures creuses'
		call	 Wr_to_EEPROM
		MOVLF	 0x37,EE_DATA		; if not already done write end of 'minutes'
		call	 Wr_to_EEPROM
		MOVLF	 0x64,EE_DATA		; if not already done write 100% 'heures creuses'
		call	 Wr_to_EEPROM
	  ENDIF_

		bcf	 PIR2,TMR3IF		; reset timer interrupt overflow
		MOVLF	 0xD8,TMR3H			; move W to TMR3H 0xD8F0 neg = 0x2710 = 10000 X 100 ns = 1 ms
		MOVLF	 0xF0,TMR3L			; move W to TMR3L
		MOVLF	 B'00000000',T3CON	; Set divide by 1
		bsf	 T3CON,TMR3ON		; Set timer ON

	    	bsf    FLAG,2			; Set ON bit 2 (display N2)

		WCGRAM 0x00,02h,04h,0Eh,11h,1Fh,10h,0Eh,0h		; write ''		;     O	; 0x02
	 	WCGRAM 0x08,08h,04h,0Eh,01h,0Fh,11h,0Fh,0h		; write ''		;    O   	; 0x04
	 	WCGRAM 0x10,04h,0Ah,00h,11h,11h,13h,0Dh,0h		; write ''		;   OOO    	; 0x0E
		WCGRAM 0x18,08h,04h,0Eh,11h,1Fh,10h,0Eh,0h		; write ''		;  0   O	; 0x11
														;  OOOOO	; 0x1F
	    	bcf    FLAG,2			; Set OFF bit 2 (display N1)			;  O		; 0x10
														;   OOO	; 0x0E
		WCGRAM 0x00,02h,04h,0Eh,11h,1Fh,10h,0Eh,0h		; write ''		;  		; 0x00
	 	WCGRAM 0x08,08h,04h,0Eh,01h,0Fh,11h,0Fh,0h		; write ''
	 	WCGRAM 0x10,04h,0Ah,00h,11h,11h,13h,0Dh,0h		; write ''
		WCGRAM 0x18,08h,04h,0Eh,11h,1Fh,10h,0Eh,0h		; write ''
		return

;========== Get Date and time =============================================================================================

Get_date	; Get all parameters from RMC sentence

		MOVLF	 d'9',Work0			; set 9th field
		call	 Get_Pointer_0100		; Get date from RMC sentence
		call	 Pack_6_bytes

		movff	 Work6,Day			; get day, month and year
		movff	 Work7,Month
		movff	 Work8,Year

		MOVLF	 d'1',Work0			; set 1st field
		call	 Get_Pointer_0100		; Get hour, minute and seconde
		call	 Pack_6_bytes

		movff	 Work6,Heure		; It's GMT time
		movff	 Work7,Minute
		movff	 Work8,Seconde

		call	 Get_GMT_time		; adjust GMT time
		call	 Get_Day_Month		; get number of day for the current month (look at year with 366 days)
		call	 Adjust_GMT			; adjust day, month if needed

	  IF_ FLAG_,2 == 0			; check if it is power ON
		call	 Write_time			; Write HH:MM:DD:MM:YY from FB to FF in EEPROM
	  	bsf	 FLAG_,2			; set FLAG_ 2 ON to say POWER ON is done
	  ENDIF_

		movff	 Year,dec2
		call	 Dec_Hex			; get Year in Hex to hex1
		movf	 hex1,W
		andlw	 b'00000011'		; keep the last 2 bits
		rlncf	 WREG
		rlncf	 WREG
		rlncf	 WREG
		rlncf	 WREG
		rlncf	 WREG
		rlncf	 WREG				; rotate them 6 times	(year is coded from 00 to 11 in the 8583 MS bits)
		iorwf	 Day				; mix year with Day for P8583
		call	 P8583_Init			; Write new time and date into P8583 I2C device
		return

;========== Check CRC in NMEA sentences ===================================================================================

Check_CRC						; process while interrupted, use specific registers

		lfsr 	 FSR2 ,0x101
		clrf	 W5

	   REPEAT_
		movff	 POSTINC2,NMEA2		; compute CRC XOR'ed each byte
		movf   NMEA2,W
		xorwf  W5
 		CMPLR	 "*",NMEA2
  	   UNTIL_	.Z.

		movlw	 "*"				; recover CRC with "*" not included
		xorwf  W5				; CRC comptuted is in W5

		movff	 POSTINC2,W1		; get CRC byte 1
		movff	 POSTINC2,W3		; get CRC byte 2

		movf	W1,W				; move CRC 1 to W
		andlw	0x0F
		btfsc	W1,6				; test if 1st CRC byte if > 0x39 (bit 6 ON)
		addlw	9				; add 9 to W if CRC 1 > 0x39
		swapf	WREG				; swap nibbles
		movwf	W1				; move W to W1

		movf	W3,W				; move CRC 2 to W
		andlw	0x0F				; remove first 4 bits
		btfsc	W3,6				; test if 2nd CRC byte if > 0x39 (bit 6 ON)
		addlw	9				; add 9 to W if CRC 2 > 0x39
		iorwf	W1				; OR W with W1

		CMPRR  W1,W5			; compare CRC computed with CRC received

	  IF_ .Z.					; if CRC'S are equal
		bcf	 FLAG,4			; FLAG 4 OFF : good receive
	  ELSE_					; else set FLAG 4 ON, bad receive
		bsf	 FLAG,4			; FLAG 4 ON : bad receive
		incf	 CRC				; incremente CRC check
	  ENDIF_

		return

;============ Sort data from RS232 ========================================================================================

Sort_NMEA_data					; GGA sentence, process while interrupted, use specific registers

		lfsr 	 FSR2 ,0x103		; Select buffer

		movff	 POSTINC2,W1
		CMPLR	 "G",W1			; compare data with "G"
		bnz	 Other_NMEA

		movff	 POSTINC2,W1
		CMPLR	 "G",W1			; compare data with "G"
		bnz	 Other_NMEA

		movff	 POSTINC2,W1
		CMPLR	 "A",W1			; compare data with "A"
		bnz	 Other_NMEA

		MOVLF	 d'7',W5			; set W5 to look at 7th field (nb of satellites)

	   REPEAT_
 		CMPLR	 ",",POSTINC2		; search for 7th field
		skpnz					; skip if not zero
		decf	 W5				; exit when the field number is reached
  	   UNTIL_ .Z.

		movff	 POSTINC2,W5
		movff	 POSTINC2,W6

		movf	 W5,W
		andlw	 0x0F
		movwf	 W3
		swapf	 W3
		movf	 W6,W
		andlw	 0x0F
		iorwf	 W3
		movff	 W3,Sat_detected
		bcf	 FLAG,7				; it is GGA sentence
		incf	 GGA
		return

Other_NMEA	; RMC sentence

		lfsr 	 FSR2 ,0x103			; Select buffer

		movff	 POSTINC2,W1
		CMPLR	 "R",W1				; compare data with "R"
		skpz						; skip if equal
		return

		movff	 POSTINC2,W1
		CMPLR	 "M",W1				; compare data with "M"
		skpz						; skip if equal
		return

		movff	 POSTINC2,W1
		CMPLR	 "C",W1				; compare data with "C"
		skpz						; skip if equal
		return

	IF_ FLAG_,0 == 0					; test if update time is allowed

		CMPLR	0x03,Month				; test if month is March

         IF_ .Z.

		   CLLRR 0x02,0x01,Heure,Minute	; Compare time with "02.01"
								; update time at 02.01 in March
            IF_ .Z.
		   bsf FLAG_,1				; allow update time for the current RMC sentence
	      ENDIF_

	   ELSE_

		   CLLRR 0x03,0x01,Heure,Minute	; Compare time with "03.01"
								; update time at 03.01 for all other months
            IF_ .Z.
		   bsf FLAG_,1				; allow update time for the current RMC sentence
	      ENDIF_

	   ENDIF_

	ENDIF_

		CLLRR 0x05,0x00,Heure,Minute		; Compare time with "05.00"

         IF_ .Z.
		bcf	FLAG_,0				; clear FLAG_,0 to allow update next day
	   ENDIF_

		bsf	 FLAG,7				; it is RMC sentence
		incf	 RMC
		return

;========== Get pointer value =============================================================================================

Get_Pointer_0100

		lfsr 	 FSR1 ,0x103		; Select buffer, enter with field number in Work0

	   REPEAT_
 		CMPLR	 ",",POSTINC1		; search for ',' character
		skpnz					; skip if not zero
		decf	 Work0			; and count them
  	   UNTIL_ .Z.

		return

;========== Pack data in Work2 and Work3 ==================================================================================

Pack_4_bytes

		movff	 POSTINC1,Work5
		movff	 POSTINC1,Work6
		movff	 POSTINC1,Work7
		movff	 POSTINC1,Work8

		movf	 Work5,W
		andlw	 0x0F
		movwf	 Work3
		swapf	 Work3
		movf	 Work6,W
		andlw	 0x0F
		iorwf	 Work3

		movf	 Work7,W
		andlw	 0x0F
		movwf	 Work2
		swapf	 Work2
		movf	 Work8,W
		andlw	 0x0F
		iorwf	 Work2
		return

;========== Store and pack ================================================================================================

Pack_6_bytes

		movff	POSTINC1,Work0
		movff	POSTINC1,Work1
		movff	POSTINC1,Work2
		movff	POSTINC1,Work3
		movff	POSTINC1,Work4
		movff	POSTINC1,Work5

		movf	Work0,W
		andlw	0x0F
		movwf	Work6
		swapf	Work6
		movf	Work1,W
		andlw	0x0F
		iorwf	Work6

		movf	Work2,W
		andlw	0x0F
		movwf	Work7
		swapf	Work7
		movf	Work3,W
		andlw	0x0F
		iorwf	Work7

		movf	Work4,W
		andlw	0x0F
		movwf	Work8
		swapf	Work8
		movf	Work5,W
		andlw	0x0F
		iorwf	Work8
		return

;============= Wait for at least 3 satellites detected ====================================================================

Wait_3_Satellites

		bcf	 FLAG,0		; Clear FLAG wait for $
		bcf	 FLAG,1		; Clear FLAG wait for *

    LOOP_
		clrwdt

	IF_ FLAG,1 == 1			; Test receive complete

		lfsr 	 FSR1 ,0x103	; Select buffer

		movff	 POSTINC1,Work1
		CMPLR	 "R",Work1		; compare data with "R"
		bnz	 Not_RMC		; Not "R"

		movff	 POSTINC1,Work1
		CMPLR	 "M",Work1		; compare data with "M"
		bnz	 Not_RMC		; Not "M"

		movff	 POSTINC1,Work1
		CMPLR	 "C",Work1		; compare data with "C"
		bnz	 Not_RMC		; Not "C"

		MOVLF	 d'2',Work0
		call	 Get_Pointer_0100
		movff	 POSTINC1,Work1
		CMPLR	 "A",Work1		; compare data with "A" RMC data valid (hour and date)

	  IF_ .Z.
		BREAK_			; exit LOOP time is now correct
	  ENDIF_

Not_RMC
		bcf	 FLAG,0		; Clear FLAG wait for $
		bcf	 FLAG,1		; Clear FLAG wait for *
	ENDIF_

    ENDLOOP_

		return

;========== Display string up to "$" ======================================================================================

LCD_variable

      LOOP_
		tblrd*+			; get table value to TABLAT
		CMPLR	 "$",TABLAT	  	; compare data with "$"
	  IF_ .Z.
		BREAK_			; exit LOOP when $ found
	  ENDIF_
		movf	 TABLAT,W
		call	 LCD_Char		; Send data to display until $ is reached
      ENDLOOP_

		return

;========== Display current temperature ===================================================================================

Display_temperature

		movf	 Degre,W			; test if Degre is < 10
		andlw	 b'01110000'

	   IF_ .Z.					; temperature is < 10
		SPACES 1
	   ENDIF_

		movlw	 "+"
		btfss	 POSITIVE,7			; Skip if positive
		movlw	 "-"
		call	 LCD_Char

		movf	 Degre,W			; test if Degre is < 10
		andlw	 b'01110000'

	   IF_ .Z. 					; temperature is < 10
		movff	 Degre,LCD_CHAR_HEXA
		call	 LCD_Single
	   ELSE_
		movf	 Degre,W			; move Degre to W
		call	 LCD_Ascii
	   ENDIF_

		call	 LCD_Point
		movff	 Degre10,LCD_CHAR_HEXA
		call	 LCD_Single
		movlw	 0xdf				; display '' character
		call	 LCD_Char
		movlw	 "C"				; display celcius
		call	 LCD_Char
		return

;========== Display Time ==================================================================================================

Display_Time

		call	 LCD_Line1

		movf	 Day,W			; test if hour is 1X or 2X or 3X
		andlw	 b'00110000'

	   IF_ .NZ.
		movf	 Day,W			; move Day to W
		call	 LCD_Ascii
	   ELSE_
		SPACES 1
		movff	 Day,LCD_CHAR_HEXA
		call	 LCD_Single
	   ENDIF_

		SPACES 1

		call   Month_data
		call   LCD_variable
		SPACES 1

		movlw	 "2"
		call	 LCD_Char
		movlw	 "0"
		call	 LCD_Char
		movf	 Year,W			; move Year to W
		call	 LCD_Ascii

		SPACES 1
		movlw	 "-"
		call	 LCD_Char
		SPACES 1

		movf	 Heure,W			; test if hour is 1X or 2X
		andlw	 b'00110000'

	   IF_ .NZ.
		movf	 Heure,W			; move Heure to W
		call	 LCD_Ascii
	   ELSE_
		SPACES 1
		movff	 Heure,LCD_CHAR_HEXA
		call	 LCD_Single
	   ENDIF_

		movlw	 "h"
		call	 LCD_Char
		movf	 Minute,W			; move Minute to W
		call	 LCD_Ascii
		movlw	 "m"
		call	 LCD_Char
		movf	 Seconde,W			; move Seconde to W
		call	 LCD_Ascii
		movlw	 "s"
		call	 LCD_Char
		SPACES 6
		return

;========== Display both temperatures =====================================================================================

Display2temp

		movlw	 0x57
		call	 LCD_Locate
		call	 Display_temperature
		SPACES 4
		call   Display_temp_prog
		return

;========== Display number of Satellites in vue ===========================================================================

Display_Sat

		call	LCD_Line2

	  IF_ Sat_detected,4 == 1	; test if bit 4 is ON 0001.0000 ( > 9 satellites)
		movf	Sat_detected,W
		call	LCD_Ascii		; display 2 digits
	  ELSE_
		movff	Sat_detected,LCD_CHAR_HEXA
		call	LCD_Single		; display 1 digit
	  ENDIF_

		POINT	Satellites		; point to string
		call	LCD_variable	; display string

		return

;========== Get Temperature in degre decimal ==============================================================================

Get_Temperature_dec

		MOVLF	 DS75_ADDR,I2C_ADDR
		call	 DS75_Read

		bsf	 POSITIVE,7			; Force positive

		call	 Adjust_DS75
;		call	 Adjust_DS75		; for test purpose only

		movff	 DegreH,DegreD
		movff	 Degre10H,Degre10D

		btfsc	 DegreH,7			; Test if Temperature is negative, skip if yes
		bcf	 POSITIVE,7

		movf	 DegreH,W			; move temperature in Hexa to W

	  IF_ .N.
		negf	 Degre10H			; negate all as temperature is negative
		skpz					; skip if zero
		incf	 DegreH
		negf	 DegreH
		movff	 DegreH,Degre
		movff	 Degre,hex1
	  ELSE_
		movwf	 hex1
	  ENDIF_

		call	 Hex_Dec			; get temperature decimal value
		movff	 dec2,Degre

		POINT  Degre_table		; point to table and search for decimal value

		; 0x0100,0x0201,0x0303,0x0404,0x0605,0x0706,0x0808,0x0909

		movf	 Degre10H,W			; move Degre10 in W
		rrncf	 WREG				; divide by 2
		rrncf	 WREG				; divide by 2
		rrncf	 WREG				; divide by 2
		rrncf	 WREG				; divide by 2 make offset to point to byte table
		addwf	 TBLPTRL			; add the offset to table pointer
		skpnc
		incf	 TBLPTRH			; incremente HIGH pointer as Carry is ON
		tblrd*				; get data from the table in TABLAT
		movff	 TABLAT,Degre10		; move value to Degre10
		return

;========== PCF8583 Init ==================================================================================================

P8583_Init

		MOVLF	P8583_ADDR,I2C_ADDR	; move W to I2C address
		bcf	I2C_ADDR,0			; write mode
		call	I2C_Setup			; gets slave attention

		WLI2C 0x00,I2C_DATA	  	; sends 8583 REG address
		WLI2C 0x00,I2C_DATA  		; clear status register
		WLI2C 0x00,I2C_DATA	  	; clear hundred of seconds
		WRI2C	Seconde,I2C_DATA  	; sends data to 8583
		WRI2C	Minute,I2C_DATA		; sends data to 8583
		WRI2C	Heure,I2C_DATA		; sends data to 8583
		WRI2C	Day,I2C_DATA		; sends data to 8583
		WRI2C	Month,I2C_DATA		; sends data to 8583
		call	Stop		  		; send stop bit
		return

;========== Send Address, Ack, GET Seconde, Minute, Heure =================================================================

P8583_Time

		MOVLF	 P8583_ADDR,I2C_ADDR
		bcf	 I2C_ADDR,0			; write mode
		call	 I2C_Setup			; sends Start, address A0, gets Ack
		WLI2C	 0x02,I2C_DATA		; sends 8583 register adress

		bsf	 I2C_ADDR,0			; read mode
		call   I2C_Setup
		call	 RdDATA
		call   Ack
		movff	 I2C_DATA,Seconde		; move I2C_DATA to Seconde
		call	 RdDATA
		call	 Ack
		movff	 I2C_DATA,Minute 		; move I2C_DATA to Minute
		call	 RdDATA
		call	 Ack
		movff	 I2C_DATA,Heure		; move I2C_DATA to Heure
		call	 RdDATA
		call	 Ack
		movff	 I2C_DATA,Day		; move I2C_DATA to Day
		movlw	 b'00111111'		; remove year bits 6 and 7
		andwf	 Day				; move to Day
		call	 RdDATA
		call	 Nack
		movff	 I2C_DATA,Month		; move I2C_DATA to Month
		movlw	 b'00011111'		; load filter to W get weekday OFF
		andwf	 Month
		call   Stop
		return

;========== Adjust DS75 ===================================================================================================

Adjust_DS75						; Used to calibrate DS75 at the real temperature
							; or put any value to check the program

	  IF_ PORTB,RB7 == 1			; RB7 is grounded in machine #2 and leaved up in machine #1
		MOVLF 0x4C,Work0			; machine #1, 0x4C = 76 X 0.0625 = 4.75C to be substract
	  ELSE_
	      MOVLF 0x38,Work0			; machine #2, 0x38 = 56 X 0.0625 = 3.50C to be substract
	  ENDIF_

		movlw	 0x10
		mulwf	 Work0			; offset X by 0x10 in PRODH and PRODL

		movf	 PRODL,W			; move offset LOW in WREG
		subwf	 Degre10H			; substract it from Degre10H
		movf	 PRODH,W			; move offset HIGH in WREG
		subwfb DegreH			; substract it from DegreH
		return

;========== Read temperature ==============================================================================================

DS75_Read
		bsf	I2C_ADDR,0			; read mode
		call 	I2C_Setup
		call	RdDATA
		call	Ack
		movff	I2C_DATA,DegreH
		call	RdDATA
		call 	Nack
		movlw	B'11110000'			; Filter Degre10
		andwf	I2C_DATA			; And I2C DATA with '11110000'
		movff	I2C_DATA,Degre10H
		bcf	SSPCON2,RCEN		; disable receive mode
		call	Stop				; end of transfert
		return

;========== DS75 Init =====================================================================================================

DS75_Init
		MOVLF	DS75_ADDR,I2C_ADDR	; move W to I2C address
		bcf	I2C_ADDR,0			; write mode
		call	I2C_Setup			; gets slave attention
		WLI2C	0x01,I2C_DATA		; sends DS75 REG address
		WLI2C	0x60,I2C_DATA		; configure DS75 in 12 bits mode
		call	Stop				; send stop bit
		DELAY	d'1'				; wait 1 ms
		call	I2C_Setup			; gets slave attention
		WLI2C	0x00,I2C_DATA		; sends DS75 REG address
		call	Stop				; send stop bit
		return

;========== Write to EEPROM ===============================================================================================

Wr_to_EEPROM

		bcf	 FLAG,0			; Set FLAG 0 OFF, $ received because RS232 can't work while writing
		movff  EE_ADDR,EEADR 		; Data Memory Address to read
		movff  EE_DATA,EEDATA		; Data Memory Value to write
		bcf 	 EECON1,EEPGD 		; Point to DATA memory
		bcf 	 EECON1,CFGS 		; Access program FLASH or Data EEPROM memory
		bsf 	 EECON1,WREN 		; Enable writes
		bcf 	 INTCON,GIE 		; Disable interrupts
	 	MOVLF  0x55,EECON2 		; Write 55h
		MOVLF  0xAA,EECON2 		; Write AAh
		bsf 	 EECON1,WR			; Set WR bit to begin write

	   REPEAT_
	   UNTIL_	 EECON1,WR == 0

		bsf 	 INTCON,GIE			; Enable interrupts
		bcf	 EECON1,WREN		; Disable writes on write complete (EEIF set)
		incf	 EE_ADDR
		incf	 Store_COUNT_L
		skpnz
		incf	 Store_COUNT_H
		lfsr 	 FSR0,0x100			; Set buffer to start
		return

;========== Read from EEPROM ==============================================================================================

Rd_from_EEPROM

		movff  EE_ADDR,EEADR 		; Data Memory Address to read
		bcf 	 EECON1,EEPGD 		; Point to DATA memory
		bcf 	 EECON1,CFGS 		; Access program FLASH or Data EEPROM memory
		bsf 	 EECON1,RD			; EEPROM Read
		movf 	 EEDATA,W 			; move EEDATA to W
		incf	 EE_ADDR
		return

;========== Write New value to thermostat =================================================================================

Write_new_thermostat

		MOVLF	 0xF0,EE_ADDR
		movff	 Work0,EE_DATA
		call	 Wr_to_EEPROM
		movff	 Work1,EE_DATA
		call	 Wr_to_EEPROM
		return

;========== GMT Reg contains the number of hours to adjust ================================================================

Get_GMT_time	;  Jan+1  Fev+1  Mar+?  Avr+2  Mai+2  Jui+2  Jul+2  Aou+2  Sep+2  Oct+?  Nov+1  Dec+1
			; 0x0101,0x0201,0x0380,0x0402,0x0502,0x0602,0x0702,0x0802,0x0902,0x1040,0x1101,0x1201

		POINT	 Month_Table
		movff	 Month,dec2
		call	 Dec_Hex
		decf	 hex1				; - 1 at Month to start the table at 0x00
		rlncf	 hex1				; multiply by 2
		movf	 hex1,W
		addwf	 TBLPTRL			; add offset to pointer LOW
		skpnc					; check if carry ON
		incf	 TBLPTRH			; if yes incremente pointer HIGH
		tblrd*+
		movff	 TABLAT,Work2		; Work2 contains GMT value (1 or 2) or bit 7 for March or bit 6 for October


		movff	 Year,dec2			; move Year to dec2
		call	 Dec_Hex			; transform it in Hex

		movlw	 d'28'			; test if Year > 28
		cpfslt hex1
		subwf	 hex1				; sub 28 from Year if > 28
		cpfslt hex1
		subwf	 hex1				; sub 28 from Year if > 28
		cpfslt hex1
		subwf	 hex1				; sub 28 from Year if > 28 (should work fine until 2099)

		rlncf	 hex1				; mutiply year offset by 2

		movf	 Work2,W			; move Work2 to WREG
		andlw	b'00000011'			; check for one of the 2 last bits ON

	   IF_ .NZ.					; if not Zero one of the 2 last bits is ON, so it's not March nor October
		 movff  Work2,GMT			; other month, so store value from Month_table to GMT
		 return
	   ENDIF_

; find the last Sunday of the month depending on the year

; March       2000*  2001   2002   2003   2004*  2005   2006   2007   2008*  2009   2010   2011   2012*  2013
;	       0x2601,0x2500,0x3100,0x3000,0x2801,0x2700,0x2600,0x2500,0x3001,0x2900,0x2800,0x2700,0x2501,0x3100
;
;		  2014   2015   2016*  2017   2018   2019   2020*  2021   2022   2023   2024*  2025   2026   2027
;	       0x3000,0x2900,0x2701,0x2600,0x2500,0x3100,0x2901,0x2800,0x2700,0x2600,0x3101,0x3000,0x2900,0x2800


; October     2000*  2001   2002   2003   2004*  2005   2006   2007   2008*  2009   2010   2011   2012*  2013
;	       0x2901,0x2800,0x2700,0x2600,0x3101,0x3000,0x2900,0x2800,0x2601,0x2500,0x3100,0x3000,0x2801,0x2700
;
;		  2014   2015   2016*  2017   2018   2019   2020*  2021   2022   2023   2024*  2025   2026   2027
;	       0x2600,0x2500,0x3001,0x2900,0x2800,0x2700,0x2501,0x3100,0x3000,0x2900,0x2701,0x2600,0x2500,0x3100
; jkl

	   IF_ Work2,7 == 1			; test bit 7, if ON it's March
		POINT	 March_Table		; point to March table
	   ELSE_
            POINT  October_Table		; point to October table
	   ENDIF_

		movf	 hex1,W
		addwf	 TBLPTRL			; add the offset (Year in Hex)
		skpnc
		incf	 TBLPTRH
		tblrd*+
		tblrd*
		movff	 TABLAT,Work4		; Work4 contains summertime or wintertime day

	   IF_ Work2,7 == 1			; test bit 7, if ON it's March **********************

		    CMPRR  Work4,Day		; Check if today is summertime day

		IF_ .Z.				; It's day to change time
		      CMPLR	 0x01,Heure

	         IF_ .Z.
		      MOVLF  2,GMT		; Set summertime (2 hours)
			return
		   ELSE_
		      MOVLF  2,GMT		; Set summertime (2 hours)
		      movlw  1			; Move 1 into WREG
		      cpfsgt Heure		; Skip if Heure greater than WREG, so keep summertime
		      decf   GMT			; As it's lower get wintertime
		      return
	         ENDIF_

		ELSE_					; It's March but not day to change time
		      MOVLF  2,GMT		; Set summertime (2 hours)
		      movf   Work4,W		; Move summertime table value into WREG
		      cpfsgt Day			; Skip if Day greater than WREG, so keep summertime
		      decf   GMT			; As it's lower get wintertime
		   return
		ENDIF_


	   ELSE_					; It's October *************************************

		   CMPRR  Work4,Day		; Check if today is wintertime day

		IF_ .Z.				; It's day to change time
		      CMPLR	 0x01,Heure

	         IF_ .Z.
		      MOVLF  1,GMT		; Set wintertime (1 hour)
			return
		   ELSE_
		      MOVLF  1,GMT		; Set wintertime (1 hour)
		      movlw  1			; Move 1 into WREG
		      cpfsgt Heure		; Skip if Heure greater than WREG, so keep summertime
		      incf   GMT			; As it's lower get summertime
		      return
	         ENDIF_

		ELSE_					; It's October but not day to change time
		      MOVLF  1,GMT		; Set wintertime (1 hour)
		      movf   Work4,W		; Move wintertime table value into WREG
		      cpfsgt Day			; Skip if Day greater than WREG
		      incf   GMT			; As it's lower get summertime
		      return
		ENDIF_

	   ENDIF_

;========== Get Number of days for a month ================================================================================

Get_Day_Month

		movff	 Year,dec2
		call	 Dec_Hex

		movlw	 d'28'			; test if Year > 28
		cpfslt hex1
		subwf	 hex1				; sub 28 from Year if > 28
		cpfslt hex1
		subwf	 hex1				; sub 28 from Year if > 28
		cpfslt hex1
		subwf	 hex1				; sub 28 from Year if > 28 (should work fine until 2099)
		rlncf	 hex1				; mutiply offset by 2

		POINT	 March_Table		; point to table
		movf	 hex1,W
		addwf	 TBLPTRL
		skpnc
		incf	 TBLPTRH
		tblrd*+
		movff	 TABLAT,Day_Month		; Day_Month contains 0 or 1 if year can be divided by 4 it is 1

;--------------------------------------------------------------------------------------------------------------------------

		POINT	 Year_Table             ; 01   02   03   04   05   06   07   08   09   10   11   12
						      ;0x31,0x28,0x31,0x30,0x31,0x30,0x31,0x31,0x30,0x31,0x30,0x31
		movff	 Month,dec2
		call	 Dec_Hex
		decf	 hex1                   ; - 1 at Month to start table at 0x00
		rlncf	 hex1				; mutiply offset by 2
		movf	 hex1,W
		addwf	 TBLPTRL
		skpnc
		incf	 TBLPTRH
		tblrd*
		movff	 TABLAT,Work8           ; Work8 contains the number of day for a month

		CMPLR	 0x02,Month			; Compare Month with 2 (February)

	  IF_ .Z.
		movf	 Work8,W			; contains the number of day for a month
		addwf	 Day_Month			; Day_Month + 0 or + 1 (february)
	  ELSE_
		movff	 Work8,Day_Month
	  ENDIF_

		return

;========== Adjust Time and Date ==========================================================================================

Adjust_GMT						; can change hour, day, month and year
		movff	 GMT,Work0			; Ie : 23h32 mn the 31/12/2006 will be 0h32 mn the 01/01/2007 with a GMT of 1

	   REPEAT_
		INCREG Heure			; incremente Heure
		movlw	 0x23
		cpfsgt Heure			; test if Heure > 0x23
		bra	 Still_More			; branch if not > 0x23
		clrf	 Heure			; it is > 0x23, so set Heure to 0x00
		INCREG Day				; incremente Day
		movf	 Day_Month,W		; Day_Month contains the number of days for the current month
		cpfsgt Day				; test if Day > Day_Month
		bra	 Still_More			; branch if not >
		MOVLF	 1,Day			; initialize Day with 1
		INCREG Month			; incremente Month
		movlw	 0x12
		cpfsgt Month			; test if Month > 0x12
		bra	 Still_More			; branch if not > 0x12
		MOVLF	 1,Month			; initialize Month with 1
		INCREG Year				; incremente Year
Still_More
		decf	 Work0			; decremente GMT value
	   UNTIL_ .Z.				; do until GMT not zero

		return

;========== Display temperature from EEPROM ===============================================================================

Display_temp_prog	; The DJU is based on 18C so add 2C to simulate normal temperature

		movf	 Degre_Rot,W
		addlw	 1					; add 1 to W  the user sees 18C as 20C
		daw						; Adjust W to decimal
		addlw	 1					; add 1 to W
		daw						; Adjust W to decimal
		call	 LCD_Hexa
		call	 LCD_Point
		movff	 Degre10_Rot,LCD_CHAR_HEXA
		call	 LCD_Single
		movlw	 0xdf					; display '' character
		call	 LCD_Char
		movlw	 "C"					; display celcius
		call	 LCD_Char
		return

;========== Display Night ==================================================================================================

Night
		call	 LCD_Line1
		POINT	 Heures_creuses		; point to table
		call	 LCD_variable

		call	 Display_start_time

		movlw	 0x01				; Display Character '' from CGRAM
		call	 LCD_Char

		SPACES 1
		call	 Display_end_time
		return

;========== Display Day_time ==============================================================================================

Day_time
		call	 LCD_Line2
		POINT	 Heures_pleines		; point to string
		call	 LCD_variable

		call	 Display_start_time2

		movlw	 0x01				; Display Character '' from CGRAM
		call	 LCD_Char

		SPACES 1
		call	 Display_end_time2

		SPACES 8
		return

;========== Display START TIME ============================================================================================

Display_start_time

		movf	 START_TIME_H,W		; test if hour is 1X or 2X
		andlw	 b'00110000'

	   IF_ .NZ.
		movf	 START_TIME_H,W
		call	 LCD_Ascii
	   ELSE_
		SPACES 1
		movff	 START_TIME_H,LCD_CHAR_HEXA
		call	 LCD_Single
	   ENDIF_

		movlw	 "h"
		call	 LCD_Char
		movf	 START_TIME_M,W
		call	 LCD_Hexa
		SPACES 1
		return

;========== Display END TIME ==============================================================================================

Display_end_time

		movf	 END_TIME_H,W		; test if hour is 1X or 2X
		andlw	 b'00110000'

	   IF_ .NZ.
		movf	 END_TIME_H,W
		call	 LCD_Ascii
	   ELSE_
		SPACES 1
		movff	 END_TIME_H,LCD_CHAR_HEXA
		call	 LCD_Single
	   ENDIF_

		movlw	 "h"
		call	 LCD_Char
		movf	 END_TIME_M,W
		call	 LCD_Hexa
		movlw	 "m"
		call	 LCD_Char
		movlw	 "n"
		call	 LCD_Char
		return

;========== Display START TIME ============================================================================================

Display_start_time2

		movf	 START_TIME_H2,W		; test if hour is 1X or 2X
		andlw	 b'00110000'

	   IF_ .NZ.
		movf	 START_TIME_H2,W
		call	 LCD_Ascii
	   ELSE_
		SPACES 1
		movff	 START_TIME_H2,LCD_CHAR_HEXA
		call	 LCD_Single
	   ENDIF_

		movlw	 "h"
		call	 LCD_Char
		movf	 START_TIME_M2,W
		call	 LCD_Hexa
		SPACES 1
		return

;========== Display END TIME ==============================================================================================

Display_end_time2

		movf	 END_TIME_H2,W		; test if hour is 1X or 2X
		andlw	 b'00110000'

	   IF_ .NZ.
		movf	 END_TIME_H2,W
		call	 LCD_Ascii
	   ELSE_
		SPACES 1
		movff	 END_TIME_H2,LCD_CHAR_HEXA
		call	 LCD_Single
	   ENDIF_

		movlw	 "h"
		call	 LCD_Char
		movf	 END_TIME_M2,W
		call	 LCD_Hexa
		movlw	 "m"
		call	 LCD_Char
		movlw	 "n"
		call	 LCD_Char
		return

;========== Display date and logs stored in EEPROM ========================================================================

Display_log

	    	bcf	 FLAG,2		; Set OFF bit 2 (display N1)
		call	 LCD_Clear
		call	 LCD_Line1
		MOVLF	 0xFB,EE_ADDR	; Time address stored in EEPROM

		clrwdt

		call	 Rd_from_EEPROM
		call	 LCD_Ascii
		movlw	 "h"
		call	 LCD_Char
		call	 Rd_from_EEPROM
		call	 LCD_Ascii

		SPACES 1

		call	 Rd_from_EEPROM
		call	 LCD_Ascii
		movlw	 "/"
		call	 LCD_Char
		call	 Rd_from_EEPROM
		call	 LCD_Ascii
		movlw	 "/"
		call	 LCD_Char
		call	 Rd_from_EEPROM
		call	 LCD_Ascii

		SPACES 1

		;Display Store_COUNT
		movf	 Store_COUNT_H,w
		call	 LCD_Hexa
		movf	 Store_COUNT_L,w
		call	 LCD_Hexa

		SPACES 1

		movf 	 DJU_H_true,w	; Display true DJU (corrected with - 12 hours basis)
		call	 LCD_Hexa		; This is an extrapolation of temperature
		movf 	 DJU_L_true,w
		call	 LCD_Hexa

		call	 LCD_Line2		; Go to LCD line 2

		;Display TEMP_24 		(cumul of temperature on 24 hours basis)

		movf 	 TEMP_24_H,w
		call	 LCD_Hexa
		movf 	 TEMP_24_L,w
		call	 LCD_Hexa
		SPACES 1

		;Display DJU_H and DJU_L

		movf 	 DJU_H,w
		call	 LCD_Hexa
		movf 	 DJU_L,w
		call	 LCD_Hexa
		SPACES 1

		;Display DURATION
		movf	 HC_H,W
		call	 LCD_Hexa
		movf 	 HC_L,W
		call	 LCD_Hexa
		SPACES 1

		;Display REMAINING
		movf 	 HP_H,W
		call	 LCD_Hexa
		movf 	 HP_L,W
		call	 LCD_Hexa
		SPACES 1

		;Display TOTAL DURATION
		movf	 TOT_H,W
		call	 LCD_Hexa
		movf	 TOT_L,W
		call	 LCD_Hexa
		LCD1	 0x5D,CRC
		LCD4   0x60,MAL,TOTAL,GGA,RMC	 ; display number of MAL and NMEA sentences
		return

;========== Get key for thermostat adjust =================================================================================

RED_KEY

	    REPEAT_
		clrwdt				; clear watchdog as this timing is unpredictable
		KEY1_2P				; look at 2 keys pressed
		skpnz
		return				; exit if 2 keys pressed
	    UNTIL_ PORTB,RB0 == 1		; Wait for key released

		DELAY	 d'25'			; Delay 25 ms
		MOVLF	 0xF0,EE_ADDR
		call	 Rd_from_EEPROM
		movwf	 Work0
		call	 Rd_from_EEPROM
		movwf	 Work1
		CLLRR	 0x25,0x00,Work0,Work1	; limit high thermostat value to 23 (see as 25C)
		skpnz
		return

		movf	 Work1,W

	  IF_ .Z.
		MOVLF	 0x05,Work1			; set to 0.5C as unit is zero
	  ELSE_
		INCREG Work0			; add 1 to W as decimal is 05
		clrf	 Work1
	  ENDIF_

		call	 Write_new_thermostat
		return

;==========================================================================================================================

BLACK_KEY

	   REPEAT_
		clrwdt				; clear watchdog as this timing is unpredictable
		KEY1_2P				; look at 2 keys pressed
		skpnz
		return				; exit if 2 keys pressed
	   UNTIL_ PORTB,RB1 == 1		; Wait for key released

		DELAY	 d'25'			; Delay 25 ms
		MOVLF	 0xF0,EE_ADDR
		call	 Rd_from_EEPROM
		movwf	 Work0
		call	 Rd_from_EEPROM
		movwf	 Work1
		CLLRR	 0x10,0x00,Work0,Work1	; limit low thermostat value to 14 (see as 16C)
		skpnz					; if not low limit, decremente temperature
		return

		movf	 Work1,W			; decremente temperature by 0.5C

	  IF_ .Z.
		MOVLF	 0x05,Work1
		clrf	 WREG
		decf	 Work0
		btfss	 STATUS,DC
		movlw	 6
		subwf	 Work0
	  ELSE_
		clrf	 Work1
	  ENDIF_

		call	 Write_new_thermostat
		return

;========== 2 secondes delay ==============================================================================================

Two_seconds						; exit if any of the 2 keys was released or 2 seconds elapsed

		clrf	 dec1				; set counter to 0xF800 (0x800 steps to reach overflow = 2048 in decimal)
		MOVLF  0xF8,dec2			; count up to 2048 ms

  LOOP_

	IF_ FLAG_,7 == 1				; test if red key active

	   IF_ PORTB,RB0 == 1
		BREAK_				; exit as red key was released
	   ENDIF_

	ELSE_						; it is black key, so test if released

	   IF_ PORTB,RB1 == 1
		BREAK_				; exit as black key was released
	   ENDIF_

	ENDIF_

		DELAY  d'1'				; delay 1 ms
		incf	 dec1				; incremente counters
		skpnz
		incf	 dec2

	   IF_ .Z.
		BREAK_				; exit after 2 seconds
	   ENDIF_

  ENDLOOP_

		return

;========== 2 buttons pressed =============================================================================================

Keys_1_2

		bcf	 FLAG_,7		; clear key select
	    	bsf    FLAG,2		; Set ON bit 2 (display N2)
		call   LCD_Clear
	    	bcf    FLAG,2		; Set OFF bit 2 (display N1)
		call   LCD_Clear
		call   LCD_Line1
		POINT  Parametres
		call   LCD_variable

		MOVLF	 0xF2,EE_ADDR
		call	 Rd_from_EEPROM
		movwf	 Work8
		call	 LCD_Ascii
		movlw	 "h"
		call	 LCD_Char
		call	 Rd_from_EEPROM
		movwf	 Work7
		call	 LCD_Ascii
		movlw	 "m"
		call	 LCD_Char
		movlw	 "n"
		call	 LCD_Char
		call	 MSG_1_2
		clrf	 Work5

	   REPEAT_					; wait for 2 keys released
		clrwdt
		KEY1_2R
	   UNTIL_ .Z.

		DELAY	 d'50'			; wait 50 ms

	LOOP_

		DELAY  d'25'

	   IF_ PORTB,RB0 == 0			; look at red key +
		 call	 inc_time			; if zero key pressed
		 call  Display_new_time
		 bsf	 FLAG_,7			; set ON red key
		 call	 Two_seconds

		   clrf  dec1

            WHILE_ PORTB,RB0 == 0		; Key + was pressed at least 2 secondes
		   btfss dec1,4			; can count up to 0x4X
		   incf  dec1
		   call  inc_time			; so speed up incremente time 5 times per seconde
		   call  Display_new_time

			IF_ dec1,4 == 1		; speed up 5 times after 6 seconds
		   	    DELAY d'20'
			ELSE_
		   	    DELAY d'100'
			ENDIF_

		ENDWHILE_

	   ENDIF_

	   REPEAT_
		clrwdt				; clear watchdog as this timing is unpredictable
	   UNTIL_ PORTB,RB0 == 1		; Wait for key released


	   IF_ PORTB,RB1 == 0			; look at black key -
		 call	 dec_time
		 call  Display_new_time
		 bcf	 FLAG_,7			; set ON black key
		 call	 Two_seconds

		   clrf  dec1

            WHILE_ PORTB,RB1 == 0		; Key + was pressed at least 2 secondes
		   btfss dec1,4			; can count up to 0x4X
		   incf  dec1
		   call  dec_time			; so speed up incremente time 5 times per seconde
		   call  Display_new_time

			IF_ dec1,4 == 1		; speed up 5 times after 6 seconds
		   	    DELAY d'20'
			ELSE_
		   	    DELAY d'100'
			ENDIF_

		ENDWHILE_

	   ENDIF_


	   REPEAT_
		clrwdt				; clear watchdog as this timing is unpredictable
	   UNTIL_ PORTB,RB1 == 1		; Wait for key released

		DELAY	 d'25'			; Delay 25 ms

	   IF_ PORTB,RB6 == 0
		BREAK_
	   ENDIF_

		clrwdt				; clear watchdog as this timing is unpredictable
		DELAY  d'25'
		incf	 Work5

	   IF_ .Z.
		BREAK_
	   ENDIF_

	ENDLOOP_

;--------------------------------------------------------------------------------------------------------------------------

		DELAY  d'25'

	   REPEAT_
		clrwdt				; clear watchdog as this timing is unpredictable
	   UNTIL_ PORTB,RB6 == 1

	    	bsf    FLAG,2		; Set ON bit 2 (display N2)
		call   LCD_Clear
	    	bcf    FLAG,2		; Set OFF bit 2 (display N1)
		call   LCD_Clear

		call   LCD_Line1
		POINT  Pourcent1
		call   LCD_variable

		call   LCD_Line2
		POINT  Pourcent2
		call   LCD_variable

		MOVLF	 0xF4,EE_ADDR	; read percentage from EEPROM
		call	 Rd_from_EEPROM
		movwf	 hex1
		movwf	 Work7
		call	 Hex_Dec
		movf	 dec1,W

        IF_ .NZ.
		MOVLF	 1,LCD_CHAR_HEXA
		call	 LCD_Single
        ENDIF_

		movf	 dec2,W
		call   LCD_Hexa
		movlw	 "%"
		call	 LCD_Char
		SPACES 1
		call	 MSG_1_2

;--------------------------------------------------------------------------------------------------------------------------

		clrf	 Work5		;

    LOOP_

		DELAY  d'25'

		CMPLR	 d'100',Work7	; prevent increment when 100 is reached

      IF_  .NZ.


	   IF_ PORTB,RB0 == 0		; look at red key +
		 clrf	  Work5
		 movlw  0x54
		 call	  LCD_Locate
		 movlw  d'5'
		 addwf  Work7		; incremente 5
		 movff  Work7,hex1
		 call	  Hex_Dec
		 movf	  dec1,W

           IF_ .NZ.
		 MOVLF  1,LCD_CHAR_HEXA
		 call	  LCD_Single
           ENDIF_

		 movf	  dec2,W
		 call   LCD_Hexa
		 movlw  "%"
		 call	  LCD_Char
		 SPACES 1
		 MOVLF  0xF4,EE_ADDR		; write at 0xF4
		 movff  Work7,EE_DATA		;
		 call	  Wr_to_EEPROM		;
	   ENDIF_

      ENDIF_

	    REPEAT_
		clrwdt				; clear watchdog as this timing is unpredictable
	    UNTIL_ PORTB,RB0 == 1		; Wait for key released

		CMPLR	 d'50',Work7		; prevent decrement when 50% reached

      IF_  .NZ.

	   IF_ PORTB,RB1 == 0			; look at black key -
		 clrf	  Work5
		 movlw  0x54
		 call	  LCD_Locate
		 movlw  d'5'
		 subwf  Work7			; decremente 5
		 movff  Work7,hex1
		 call	  Hex_Dec
		 movf	  dec1,W

           IF_ .NZ.
		 MOVLF  1,LCD_CHAR_HEXA
		 call	  LCD_Single
           ENDIF_

		 movf	  dec2,W
		 call   LCD_Hexa
		 movlw  "%"
		 call	  LCD_Char
		 SPACES 1
		 MOVLF  0xF4,EE_ADDR		; write new % at 0xF4
		 movff  Work7,EE_DATA		;
		 call	  Wr_to_EEPROM		;
	   ENDIF_

      ENDIF_

	    REPEAT_
		clrwdt				; clear watchdog as this timing is unpredictable
	    UNTIL_ PORTB,RB1 == 1		; Wait for key released

		DELAY	 d'25'			; Delay 25 ms


	   IF_ PORTB,RB6 == 0
		BREAK_
	   ENDIF_

		clrwdt				; clear watchdog as this timing is unpredictable
		DELAY  d'25'
		incf	 Work5

	   IF_ .Z.
		BREAK_
	   ENDIF_


    ENDLOOP_

	    	bsf    FLAG,2		; Set ON bit 2 (display N2)
		call   LCD_Clear
	    	bcf    FLAG,2		; Set OFF bit 2 (display N1)
		call   LCD_Clear
		return

;========== Get 2 buttons value in Degre ==================================================================================

Get_thermo

		KEY1_2P			; look at 2 keys pressed

   	  IF_ .Z.				; 2 keys pressed
		call	 Keys_1_2
        ELSE_
		btfss	 PORTB,RB0		; look at red key +
		call	 RED_KEY
		btfss	 PORTB,RB1		; look at black key -
		call	 BLACK_KEY
		MOVLF	 0xF0,EE_ADDR	; read thermostat temperature
		call	 Rd_from_EEPROM
		movwf	 Degre_Rot
		call	 Rd_from_EEPROM
		movwf	 Degre10_Rot
        ENDIF_

  		return

;========== Display Msg 1 and Msg2 on second display ======================================================================

MSG_1_2

	    	bsf    FLAG,2		; Set ON bit 2 (display N2)
		call   LCD_Line1
		POINT  Msg1
		call   LCD_variable

		call   LCD_Line2
		POINT  Msg2
		call   LCD_variable
	    	bcf    FLAG,2		; Set OFF bit 2 (display N1)
		return

;========== Write time to EEPROM ==========================================================================================

Write_time

		MOVLF	 0xFB,EE_ADDR
		clrwdt
		movff	 Heure,EE_DATA
		call	 Wr_to_EEPROM	; EE_ADDR comes back with + 1
		movff	 Minute,EE_DATA
		call	 Wr_to_EEPROM	; EE_ADDR comes back with + 1
		movff	 Day,EE_DATA
		call	 Wr_to_EEPROM	; EE_ADDR comes back with + 1
		movff	 Month,EE_DATA
		call	 Wr_to_EEPROM	; EE_ADDR comes back with + 1
		movff	 Year,EE_DATA
		call	 Wr_to_EEPROM	; EE_ADDR comes back with + 1
		return

;==========================================================================================================================

Display_new_time

		clrf	 Work5			; clear Work5 (reinitilize time_out value as a key was pressed)
		movlw  0x19
		call	 LCD_Locate
		movf   Work8,W
		call	 LCD_Ascii
		movlw  "h"
		call	 LCD_Char
		movf   Work7,W
		call	 LCD_Ascii
		movlw  "m"
		call	 LCD_Char
		movlw  "n"
		call	 LCD_Char
		MOVLF  0xF2,EE_ADDR		; write new end of 'heures creuses' at 0xF2
		movff  Work8,EE_DATA		;
		call	 Wr_to_EEPROM		;
		movff  Work7,EE_DATA		;
		call	 Wr_to_EEPROM		;
		return

;==========================================================================================================================

inc_time	; used for updating end of 'heures creuses'

		INCREG Work7				; Incremente Minute
		movlw	 0x60
		cpfseq Work7				; compare W with Minutes, skip if equal
		return					; not equal just return

		clrf	 Work7				; Minutes are '60'
		INCREG Work8				; Incremente Hours
		movlw	 0x24
		cpfseq Work8				; compare W with Hours, skip if equal
		return					; not equal, so just return

		clrf	 Work8				; Set hours to zero as it is 24.00
		return

;========== Decremente TIME hours and minutes =============================================================================

dec_time
		decf	 Work7			; Decremente Minute
		btfsc	 STATUS,DC
		return

      IF_ .N.
		MOVLF	 0x59,Work7			; set 59 to minute
		decf	 Work8			; decremente Hours
		btfsc	 STATUS,DC
		return

	   IF_ .N.
		MOVLF	 0x23,Work8			; Set Hours to 23 as it is 00:00
	   ELSE_
		movlw	 6				; Move 6 to W
		subwf	 Work8			; Substract from Hours
         ENDIF_

      ELSE_
		movlw	 6
		subwf	 Work7
      ENDIF_

		return

;==========================================================================================================================

Inc_Time
		INCREG START_TIME_M2			; Incremente Minute
		movlw	 0x60
		cpfseq START_TIME_M2
		return

		clrf	 START_TIME_M2
		INCREG START_TIME_H2			; Incremente Hours
		movlw	 0x24
		cpfseq START_TIME_H2
		return

		clrf	 START_TIME_H2			; Set hours to zero as it is 24.00
		return

;========== Decremente TIME hours and minutes =============================================================================

Dec_Time
		decf	 Work0			; Decremente Minute
		btfsc	 STATUS,DC
		return

      IF_ .N.
		MOVLF	 0x59,Work0			; set 59 to minute
		decf	 Work1			; decremente Hours
		btfsc	 STATUS,DC			; test DC, skip if not DC
		return

	   IF_ .N.
		MOVLF	 0x23,Work1			; Set Hours to 23 as it is 00:00
	   ELSE_
		movlw	 6				; Move 6 to W
		subwf	 Work1			; Substract from Hours
	   ENDIF_

      ELSE_
		movlw	 6
		subwf	 Work0
      ENDIF_

		return

;========== Get Starting Time Heures creuses in hours and minutes =========================================================

Get_Start_Time_HC

		bcf	 FLAG_,5			; clear FLAG_5
		call	 Get_E_Time
		movff	 HC_H,Work2
		movff	 HC_L,Work3

		negf	 Work3			; negate time value
		skpz					; skip if zero
		incf	 Work2
		negf	 Work2

	   WHILE_ .NZ.				; do it until time is not zero
		incf	Work3
		skpnz
		incf	Work2
		call  Dec_Time
		CRrRr	Heure,Work1,Minute,Work0
		skpnz
		bsf	FLAG_,5			; set FLAG_ 5 ON time is within start and end period
		TSTZ  Work2,Work3			; is time zero ?
  	   ENDWHILE_

		movff	 Work0,START_TIME_M	; update START_TIME for heures creuses
		movff	 Work1,START_TIME_H
		call	 Get_E_Time
		movff	 Work0,END_TIME_M		; recover END_TIME for heures creuses
		movff	 Work1,END_TIME_H
		return

;========== Get Ending Time heures pleines in hours and minutes ===========================================================

Get_Start_Time_HP

		bcf	 FLAG_,6
		call	 Get_S2_Time
		movff	 HP_H,Work2
		movff	 HP_L,Work3

		clrc						; Clear Carry
		clrdc						; Clear DC

		rrcf	 Work2				; divide time 'heures pleines' by 2
		rrcf	 Work3
		bcf	 Work2,7				; Clear MS bit
		negf	 Work3
		skpz						; skip if zero
		incf	 Work2
		negf	 Work2

	   WHILE_ .NZ.					; do it until time is not zero
		incf	Work3
		skpnz
		incf	Work2
		call  Inc_Time
		TSTZ  Work2,Work3				; is time zero ?
  	   ENDWHILE_

		movff	 START_TIME_H2,END_TIME_H2	; update START and END TIME
		movff	 START_TIME_M2,END_TIME_M2	;
		movff	 START_TIME_H2,Work1		; update Work1 and Work0
		movff	 START_TIME_M2,Work0		;
		call	 Second_part

		movf	 FLAG_,W
		andlw	 b'01100000'			; test if FLAG_,5 or FLAG_6 is ON

         IF_ .NZ.
		bsf	 LATE,RE0				; POWER ON as one of the 2 FLAG'S or both are ON
		bsf	 LATE,RE1				; Led ON
	   ELSE_
		bcf	 LATE,RE0				; POWER OFF as both FLAG'S are OFF
		bcf	 LATE,RE1				; Led OFF
	   ENDIF_

		return

;========== Second part search for starting time heures pleines ===========================================================

Second_part

		movff	 HP_H,Work2
		movff	 HP_L,Work3
		negf	 Work3
		skpz					; skip if zero
		incf	 Work2
		negf	 Work2

	   WHILE_ .NZ.				; do it until time is not zero
		incf	Work3
		skpnz
		incf	Work2
		call  Dec_Time
		CRrRr	Heure,Work1,Minute,Work0
		skpnz
		bsf	FLAG_,6			; set FLAG_ 6 ON time is within start and end period
		TSTZ  Work2,Work3			; is time zero ?
  	   ENDWHILE_

		movff	 Work1,START_TIME_H2	; update START and END TIME for heures pleines
		movff	 Work0,START_TIME_M2	;
		return

;========== Compute DJU value =============================================================================================

Compute_DJU
		clrf	 Work0
		clrf	 Work1
		clrf	 Work6
		clrf	 Work7
		clrf	 TEMP_24_H			; Clear TEMP_24_H
		clrf	 TEMP_24_L			; Clear TEMP_24_L
		clrf	 EE_ADDR			; Clear EE_ADDR
		movff	 DegreH,Work2		; Save DegreH to Work2
		movff	 Degre10H,Work3		; Save Degre10H to Work3

      REPEAT_
		call	 Rd_from_EEPROM
		movwf	 DegreH
		call	 Rd_from_EEPROM
		movwf	 Degre10H
		movf	 DegreH,W

	  IF_ .N.					; test if negative
		negf	 Degre10H
		skpz					; skip if zero
		incf	 DegreH
		negf	 DegreH
		swapf	 Degre10H

		movlw	 0x0A				; Load W with X 10
		mulwf	 DegreH			; Muliply Degre with 10
		movf	 PRODL,W			; Move LOW to W
		subwf	 TEMP_24_L			; sub LOW from TEMP_24_L
		movf	 PRODH,W			; move HIGH to W
		subwfb TEMP_24_H			; sub it from TEMP_24_H

		movlw	 0x0A
		mulwf	 Degre10H			; multiply by 10
		movf	 PRODL,W			; move LOW to W
		addwf	 Work6			; Add it to Work6
		movf	 PRODH,W
		addwfc Work7			; Add HIGH to Work7 with carry
	  ELSE_
		movlw	 0x0A				; Load W with X 10
		mulwf	 DegreH			; Muliply Degre with 10
		movf	 PRODL,W			; move LOW to W
		addwf	 TEMP_24_L			; Add it to TEMP_24_L
		movf	 PRODH,W			; Move HIGH to W
		addwfc TEMP_24_H			; Add HIGH to TEMP_24_H with carry

		swapf	 Degre10H
		movlw	 0x0A
		mulwf	 Degre10H
		movf	 PRODL,W			; move LOW to W
		addwf	 Work0			; Add it to Work0
		movf	 PRODH,W
		addwfc Work1			; Add HIGH to Work1 with carry
	  ENDIF_

		CMPLR  0x60,EE_ADDR		; Compare EE_ADDR with 60
	UNTIL_ .Z.

		clrc					; Clear Carry
		clrdc					; Clear DC

		MOVLF	4,Work8			; set counter to 4 (prepare to divide by 16)

	   REPEAT_					; Divide by 16 the result in Work1 and Work0
		rrcf	 Work1			; divide by 2
		rrcf	 Work0
		bcf	 Work1,7			; clear rotating bit
		decf	 Work8			; decremente counter
	   UNTIL_ .Z.				; repeat until counter is not zero

 		movf	 Work0,W
		addwf	 TEMP_24_L
		clrf	 WREG
		addwfc TEMP_24_H
		movf	 Work1,W
		addwf	 TEMP_24_H

		clrc					; Clear Carry
		clrdc					; Clear DC

		MOVLF	4,Work8

	   REPEAT_					; Divide by 16 the result in Work7 and Work6
		rrcf	 Work7
		rrcf	 Work6
		bcf	 Work7,7
		decf	 Work8
	   UNTIL_ .Z.

 		movf	 Work6,W
		subwf	 TEMP_24_L
		movf	 Work7,W
		subwfb TEMP_24_H

		movff	 Degre_Rot,Work4		; Save Degre_Rot to Work4
		movff	 Degre_Rot,dec2		; prepare dec2 to convert to hex (ie : 18C in enter)
		call	 Dec_Hex			; convert dec2 to hex1		   (back with 0x12)
		movff	 hex1,Degre_Rot		; get hex1 to Degre_Rot in hex

		movlw	 0x0A				; Load W with X 10		   (0x12  X 0x0A = 0xB4)
		mulwf	 Degre_Rot			; Muliply Degre_Rot with 10
		movff	 PRODL,Degre_Rot

		movlw	 0x05
		cpfseq Degre10_Rot		; Compare with 0, skip if 0
		bra	 It_is_zero			;
		movlw	 0x05
		addwf	 Degre_Rot			; Add 0.5C				   (add 0x05 = 0xB9)
It_is_zero
		movlw	 0x30				;					   (0xB9  X 0x30 = 0x22B0)
		mulwf	 Degre_Rot			;					   (0x22B0 = 8880 in dec)
		movff	 PRODH,DJU_H		;					   (8880/480 = 18.5C)
		movff	 PRODL,DJU_L		; so, this routine multiply the temperature by 480
							; in tenth of degre and 48 measures in 24 hours
		movf	 TEMP_24_L,W		; Move TEMP_24_L to W
		subwf	 DJU_L			; Substract W from DJU_L
		movf	 TEMP_24_H,W		; Move TEMP_24_H to W
		subwfb DJU_H			; Substract W from DJU_H

		movff	 Work4,Degre_Rot		; Restore Degre_Rot from Work4
		movff	 Work2,DegreH		; Restore Degre from Work2
		movff	 Work3,Degre10H		; Restore Degre10 from Work3

		call	 Get_12_DJU
		call	 Get_duration
		call	 Percent			; get % of total time in RH and RL
		call	 Optimize
		return

;========== optimize time HC and HP depending on percent setting ==========================================================

Optimize	; enter this with total time (TOT_H and TOT_L) and total time computed with % (RH and RL)

		movff  RH,HC_H		; move total time computed value with % to 'Heures creuses'
		movff  RL,HC_L

		movff  TOT_H,Work1	; prepare work registers with Total time
		movff  TOT_L,Work0

		movf   RL,W             ; substract RH and RL from Work1 and Work0 (Total time)
		subwf  Work0            ; this is the time computed with % ie: for 95% this is the 5 %
		movf   RH,W
		subwfb Work1

		movff	 Work1,HP_H       ; move these values to 'Heures pleines'
		movff	 Work0,HP_L

		MOVLF  0x01,Work6		; compare 0x1E0 (8h00) with 'Heures creuses'
		MOVLF	 0xE0,Work5		;
		movff	 HC_H,PRODH		;
		movff	 HC_L,PRODL
		call	 CMP16		; WREG is back with bit 0, bit 1 or bit 2 ON


  	  IF_ WREG,1 == 1			; time 'Heures creuses'  > than 0x1E0 (8 hours)
	      movlw  0xE0             ; substract 0x1E0 from 'Heures creuses' (480 mn or 8 hours)
	      subwf  HC_L
	      movlw  0x01
	      subwfb HC_H
	      movf   HC_L,W           ; and add it to 'Heures pleines'
	      addwf  HP_L
	      movf   HC_H,W
	      addwfc HP_H
	      MOVLF  0x01,HC_H		; set 'Heures creuses' to 0x1E0 (480 mn 8 hours)
	      MOVLF  0xE0,HC_L
  	  ENDIF_


	      MOVLF  0x00,Work6	      ; compare 0x01E (30 mn) with 'Heures pleines'
	      MOVLF	 0x1E,Work5
	      movff	 HP_H,PRODH
	      movff	 HP_L,PRODL
	      call	 CMP16		; WREG is back with bit 0, bit 1 or bit 2 ON

						; enter this only if 'Heures pleines' is less than 30 mn
        IF_ WREG,2 == 1			; HP time < than 0x01F (30mn)
	      movf   HP_L,W		; move 'Heures pleines'
	      addwf  HC_L			; and add it to 'Heures creuses'
	      movf   HP_H,W		;
	      addwfc HC_H			;
	      clrf   HP_L			; now clear 'Heures pleines' the value is less than 30mn
	      clrf   HP_H
        ENDIF_

	      return

;========== Get the DJU value 12 hours before and add it to real DJU ======================================================

Get_12_DJU
		call	 Get_EE_ADDR		; Get EEPROM address function of the actual time
		movlw	 0x30
		cpfsgt Minute			; Skip if greater than 30
		bra	 Not_more
		incf	 EE_ADDR			; Incremente EEPROM address as Minutes are between
		incf	 EE_ADDR			; 31 and 59

Not_more
		movlw	 0x60				; Add 0x60 to go to DJU area
		addwf	 EE_ADDR			;
		movlw	 d'48'			; Substract 48 values so 12 hours
		subwf	 EE_ADDR			;
		movlw	 0x60
		cpfslt EE_ADDR			; Test if result less than start of DJU
		bra	 Not_less
		movlw	 0x60				; If less, add 0x60
		addwf	 EE_ADDR
Not_less
		call	 Rd_from_EEPROM
		movwf	 Work0
		call	 Rd_from_EEPROM
		movwf	 Work1
		movff	 DJU_H,Work2
		movff	 DJU_L,Work3
		movff	 DJU_L,DJU_L_true
		movff	 DJU_H,DJU_H_true
		movf	 Work1,W			; Move Work1 to W
		subwf	 Work3			; Substract W from F
		movf	 Work0,W			; Move Work0 to W
		subwfb Work2			; Substract W from F
		movf	 Work3,W
		addwf	 DJU_L_true
		movf	 Work2,W
		addwfc DJU_H_true
		return

;========== Get from DJU Table duration of activation in 2 Hex bytes ======================================================

Get_duration ; DJU contains value in Hex

		POINT	 DJU_Table			; point to DJU table

		movff	 DJU_L_true,DJU_L2
		movff	 DJU_H_true,DJU_H2
		movf	 DJU_H2,W

	  IF_ .N.
		clrf	 DJU_H2			; Clear DJU_H2 if yes
		clrf	 DJU_L2			; Clear DJU_L2 if yes
        ENDIF_

		; Round the value of DJU_H2 if DJU_L2 > 0x80

		movlw	 0x80				; Move 0x80 to W
		cpfslt DJU_L2			; Compare DJU_L with W, skip if DJU_L < 0x80
		incf	 DJU_H2			; Incremente DJU_H as DJU_L > 0x80
		movlw	 0x2C
		cpfslt DJU_H2			; Compare DJU_H2 with 2C, skip if lower
		movwf	 DJU_H2			; if equal or greater force max value to 0x2C (last entry in the table)
							; in case of greater value, limit to the last entry
		rlncf	 DJU_H2			; Multiply DJU by 2
		movf	 DJU_H2,W			; Move DJU to W

		addwf	 TBLPTRL			; Add it to pointer LOW
		skpnc
		incf	 TBLPTRH			; as carry is ON incremente HIGH pointer

		tblrd*+
		movff  TABLAT,TOT_L
		tblrd*+
		movff  TABLAT,TOT_H

		movff	 TOT_H,HC_H			; get total time
		movff	 TOT_L,HC_L

		MOVLF	 0xF4,EE_ADDR		; read percentage from EEPROM
		call	 Rd_from_EEPROM
		movwf	 hex1

		movlw	 d'50'
		cpfsgt hex1				; compare with 50%
		bra	 Fifty
		addlw	 d'5'

		cpfsgt hex1				; compare with 55%
		bra	 Fifty5
		addlw	 d'5'

		cpfsgt hex1
		bra	 Sixty			; compare with 60%
		addlw	 d'5'

		cpfsgt hex1
		bra	 Sixty5			; compare with 65%
		addlw	 d'5'

		cpfsgt hex1
		bra	 Seventy			; compare with 70%
		addlw	 d'5'

		cpfsgt hex1
		bra	 Seventy5			; compare with 75%
		addlw	 d'5'

		cpfsgt hex1
		bra	 Heighty			; compare with 80%
		addlw	 d'5'

		cpfsgt hex1
		bra	 Heighty5			; compare with 85%
		addlw	 d'5'

		cpfsgt hex1
		bra	 Ninety			; compare with 90%
		addlw	 d'5'

		cpfsgt hex1
		bra	 Ninety5			; compare with 95%

		bra 	 Hundred			; assume 100%

Fifty		MUL16	TOT_H,TOT_L,d'50'
		return

Fifty5	MUL16	TOT_H,TOT_L,d'55'
		return

Sixty		MUL16	TOT_H,TOT_L,d'60'
		return

Sixty5	MUL16	TOT_H,TOT_L,d'65'
		return

Seventy	MUL16	TOT_H,TOT_L,d'70'
		return

Seventy5	MUL16	TOT_H,TOT_L,d'75'
		return

Heighty	MUL16	TOT_H,TOT_L,d'80'
		return

Heighty5	MUL16	TOT_H,TOT_L,d'85'
		return

Ninety	MUL16	TOT_H,TOT_L,d'90'
		return

Ninety5	MUL16	TOT_H,TOT_L,d'95'
		return

Hundred 	MUL16	TOT_H,TOT_L,d'100'
		return

;========== Compute offset in EEPROM fonction of the real time ============================================================

Get_EE_ADDR

		clrf	 EE_ADDR
		clrf	 Work0		; Clear Work0
		CMPRR	 Heure,Work0

	   WHILE_ .NZ.			; do it until not zero
		INCREG Work0		; incremente Work0 in decimal
		incf	 EE_ADDR		; EE_ADDR +1
		incf	 EE_ADDR		; EE_ADDR +1
		incf	 EE_ADDR		; EE_ADDR +1
		incf	 EE_ADDR		; EE_ADDR +1
		CMPRR	 Heure,Work0
  	   ENDWHILE_

		CMPLR	 0x30,Minute

	   IF_ .Z.
		incf	 EE_ADDR		; EE_ADDR +1
		incf	 EE_ADDR		; EE_ADDR +1
	   ENDIF_

		return

;========== Get END TIME ==================================================================================================

Get_E_Time

		MOVLF	 0xF2,EE_ADDR			; End time 'heures creuses' stored in EEPROM
		call	 Rd_from_EEPROM
		movwf	 Work1
		call	 Rd_from_EEPROM
		movwf	 Work0
		return

;========== Get START TIME ================================================================================================

Get_S2_Time							; add 8 hours to END TIME heures creuses to get the middle point
								; for 'heures pleines'
		call	 Get_E_Time
		movff	 Work1,START_TIME_H2
		movff	 Work0,START_TIME_M2

		MOVLF  0x01,Work3				; 0x1E0 = 480 mn = 8 hours
		MOVLF  0xE0,Work2

		negf	 Work2
		skpz						; skip if zero
		incf	 Work3
		negf	 Work3

	   WHILE_ .NZ.					; do it until time is not zero
		incf	Work2
		skpnz
		incf	Work3
		call  Inc_Time
		TSTZ  Work2,Work3				; is time zero ?
  	   ENDWHILE_

		return

;========== Check EEPROM & initialize if 2nd byte is 0xFF =================================================================

Check_EE
		MOVLF	 0x01,EE_ADDR		; Set EEPROM address to 0x01
		call	 Rd_from_EEPROM
		movlw	 0xFF
		cpfseq EEDATA			; Compare EE_DATA with FF skip if =
		return				; EEPROM OK
							; go here only the first POWER ON when the PIC is not initialized
		clrf	 EE_ADDR			; Clear EE_ADDR

	   REPEAT_
		clrwdt
		call	 Write_Degre
		CMPLR	 0x60,EE_ADDR		; Compare EE_ADDR with 0x60
	   UNTIL_ .Z.

		call	 Get_thermo
		call	 Compute_DJU		; Compute DJU
		MOVLF	 0x60,EE_ADDR

	   REPEAT_
		clrwdt
		call	 Write_DJU
		CMPLR	 0xC0,EE_ADDR		; Compare EE_ADDR with 0xC0
	   UNTIL_ .Z.

		clrf	 Store_COUNT_H
		clrf	 Store_COUNT_L
		return

;========== Store DJU to EEPROM ===========================================================================================

Write_DJU
		movff	 DJU_H,EE_DATA
		call	 Wr_to_EEPROM
		movff	 DJU_L,EE_DATA
		call	 Wr_to_EEPROM
		bcf	 FLAG,3		; Clear bit 3 to say Storing is done
		return

;========== Store temperature to EEPROM ===================================================================================

Degre_to_EE
		tstfsz Minute		; test if Minute is 00 skip if =
		bra	 next
Write_EE
		call	 Get_EE_ADDR
Write_Degre
		movff	 DegreD,EE_DATA
		call	 Wr_to_EEPROM
		movff	 Degre10D,EE_DATA
		call	 Wr_to_EEPROM
		bcf	 FLAG,3		; Clear bit 3 to say Storing is done
		return

next
		movlw	 0x30
		cpfseq Minute		; Compare Minute with 30 skip if =
		return			; Not 00 nor 30
		bra	 Write_EE

;========== Write DJU to EEPROM ===========================================================================================

DJU_to_EE
		tstfsz Minute		; test if Minute is 00 skip if =
		bra	 next_DJU
DJU_EE
		call	 Get_EE_ADDR
		movlw	 0x60
		addwf	 EE_ADDR
		movff	 DJU_H,EE_DATA
		call	 Wr_to_EEPROM
		movff	 DJU_L,EE_DATA
		call	 Wr_to_EEPROM
		bcf	 FLAG,3		; Clear bit 3 to say Storing is done
		return
next_DJU
		movlw	 0x30
		cpfseq Minute		; Compare Minute with 30 skip if =
		return			; Not 00 nor 30
		bra	 DJU_EE

;========== Test if Minute is 00 or 30, active Flag if not ================================================================

Test_00_30
		tstfsz Minute		; test if Minute is 00 skip if =
		bra	 Next1
		return
Next1
		movlw	 0x30
		cpfseq Minute		; Compare Minute with 30 skip if =
		bsf	 FLAG,3		; set bit 3 ON to say not 00 nor 30 mn
		return

;========== Convert DEC to HEX (one Byte) =================================================================================
;   enter with dec value in dec2 and this value is converted in hex1 (works OK from 00 to 99)
;==========================================================================================================================

Dec_Hex
		clrc
		movf	 dec2,W
		andlw	 0xF0
		movwf	 hex1
		swapf	 hex1
		movf	 dec2,W
		andlw	 0x0F
		movwf	 dec1

		movf   hex1,W
  		rlcf   hex1
  		rlcf   hex1
   		addwf  hex1
		movf   dec1,W
   		rlcf   hex1
   		addwf  hex1
		return

;========== Convert HEX to DEC (one Byte) =================================================================================
;   enter with hex value in hex1 and this value is converted in dec1 and dec2 (works OK from 0x00 to 0xC7) 0 to 199
;==========================================================================================================================

Hex_Dec
		clrf   dec1
            clrf   dec2
            movlw  8            	; Bit count
            movwf  Work8
loop        rlcf   hex1     		; Shift msb into
            rlcf   dec2       	; first digit
            movlw  d'10'        	; Digit > 9 ?
            subwf  dec2,W
            skpnc
            movwf  dec2         	; Digit = digit -10
            rlcf   dec1       	; Shift carry in
            movlw  d'10'        	; Digit > 9 ?
            subwf  dec1,W
            skpnc
            movwf  dec1          	; Digit = digit -10
            decfsz Work8	     	; Next
bit
            bra    loop
		swapf	 dec1,W		; swap dec1 and move it to WREG
		clrf	 dec1			; dec1 set to 0
		iorwf	 dec2			; or WREG with dec2 into dec2
		skpc
		return

		MOVLF	1,dec1		; as there is carry ON, set dec1 to 1
		return

;========== Wait 44 s ====================================================================================================

Wait44us
		MOVLF	 d'220',Work0	; initialize d'220' counter
		decfsz Work0		; decremente and test
		bra	 $-2		; loop until <> 0 - (0.1*2*220 = 44 s)
		movf	 LCD_TEMP, W	; restore W with old value
		return

;========== Wait 550S ====================================================================================================

Wait550us	; Each count is 0.1 s X 5500 X 1 = 550 s (5500 = 0x157C Neg = 0xEA84)

		MOVLF	 0xEA,TMR1H			; move W to TMR1H
		MOVLF	 0x84,TMR1L			; move W to TMR1L
		MOVLF	 B'00000000',T1CON	; divide by 1
		bsf	 T1CON, TMR1ON		; timer ON

		btfss	 PIR1, TMR1IF		; test  for timer overflow
		bra	 $-2				; if not test again
		bcf	 PIR1, TMR1IF		; reset timer overflow
		bcf	 T1CON, TMR1ON		; timer OFF
		return

;==========================================================================================================================

Delay_	; Each count is 0.1 s X 10000 X 1 = 1000 s = 1 ms (10000 = 0x2710 Neg = 0xD8F0)

		clrwdt
		movwf  Work0

	   REPEAT_
		MOVLF	 0xD8,TMR1H			; move W to TMR1H
		MOVLF	 0xF0,TMR1L			; move W to TMR1L
		MOVLF	 B'00000000',T1CON	; divide by 1
		bsf	 T1CON, TMR1ON		; timer ON
		btfss	 PIR1, TMR1IF		; test  for timer overflow
		bra	 $-2				; if not test again
		bcf	 PIR1, TMR1IF		; reset timer overflow
		bcf	 T1CON, TMR1ON		; timer OFF
		decf	 Work0
	   UNTIL_ .Z.

		return

;========== LCD Init ======================================================================================================

LCD_Init
		clrf	 LCD_DATA_PORT		; clear LCD_DATA_PORT
		clrf	 LCD_DATA_TRIS		; clear output port
		DELAY	 2				; 2 ms delay

		; system sends 3 times in 8 bits mode, then 2 times in 4 bits mode to be sure it works
		movlw	 B'00110000'		; Mode 8 bit DL ON
		call	 LCD_Send
		call	 LCD_Send
		call	 LCD_Send
		movlw	 B'00100000'		; Mode 4 bit DL off
		call	 LCD_Send			; 2 times
		call	 LCD_Send
		movlw	 B'11000000'		; 2 lines display and 5X11 dots format
		call	 LCD_Send

		; Set ON/OFF
		movlw	 B'00000000'
		call	 LCD_Send
		movlw	 B'11000000'		; Display on, cursor off, blink off
		call	 LCD_Send
		return

;========== Send data to LCD, data in W ===================================================================================

LCD_SEND_NO_WAIT					; validate data to display 1 or 2

		movwf	 LCD_DATA_PORT		; move W into PORTD
		nop

	  IF_ FLAG,2 == 0
		bsf	 LCD_DATA_PORT,LCD_E1	; raise E1
		nop
		nop
		nop
		nop
		nop					; 500 ns delay
		bcf	 LCD_DATA_PORT,LCD_E1	; drop E1
	  ELSE_
		bsf	 LCD_DATA_PORT,LCD_E2	; raise E2
		nop
		nop
		nop
		nop
		nop					; 500 ns delay
		bcf	 LCD_DATA_PORT,LCD_E2	; drop E2
	  ENDIF_

		return

;========== Send data to LCD, data in W ===================================================================================

LCD_Send
		call	 LCD_SEND_NO_WAIT
		call	 Wait44us
		return

;========== Send ASCII charactere on the LCD ===============================================================================

LCD_Char
		movwf	 LCD_CHAR			; move W in LCD_CHAR
		andlw	 B'11110000'		; mask the 4 LS bits
		iorlw	 LCD_RS			; raise RS
		call	 LCD_SEND_NO_WAIT		; Send to display
		swapf	 LCD_CHAR, W		; swap charactere
		andlw	 B'11110000'		; mask the 4 LS bits
		iorlw	 LCD_RS			; raise RS
		call	 LCD_Send			; Send to display
		return

;========== Clear LCD screen ==============================================================================================

LCD_Clear
		movlw	 B'00000000'
		call	 LCD_SEND_NO_WAIT
		movlw	 B'00010000'
		call	 LCD_Send
		DELAY	 1
		call	 Wait550us
		return

;========== LOCATE LCD Sreen ==============================================================================================

LCD_Locate
		movwf	 LCD_CHAR	 		; store W in LCD_CHAR
		andlw	 B'11110000' 		; mask less significant bits
		iorlw	 B'10000000'		; raise address DDRAM
		call	 LCD_SEND_NO_WAIT  	; Send to display
		swapf	 LCD_CHAR, W  		; put back LCD_CHAR in W swapping nibbles
  		andlw	 B'11110000' 		; mask less significant bits
		call	 LCD_Send	 		; Send to display
		return

;========== LOCATE LCD Sreen ==============================================================================================

LCD_CGRAM
		movwf	 LCD_CHAR	 		; store W in LCD_CHAR
		andlw	 B'11110000' 		; mask less significant bits
		iorlw	 B'01000000'		; raise address CGRAM
		call	 LCD_SEND_NO_WAIT  	; Send to display
		swapf	 LCD_CHAR, W  		; put back LCD_CHAR in W swapping nibbles
  		andlw	 B'11110000' 		; mask less significant bits
		call	 LCD_Send	 		; Send to display
		return

;========== Locate cursor on 1st line =====================================================================================

LCD_Line1
		movlw	 0x00
		call	 LCD_Locate
		return

;========== Locate cursor on 2nd line =====================================================================================

LCD_Line2
		movlw	 0x40
		call	 LCD_Locate
		return

;========== Display one space =============================================================================================

LCD_Space
		movlw	 " "
		call	 LCD_Char
		return

;========== Display one point =============================================================================================

LCD_Point
		movlw	 "."
		call	 LCD_Char
		return

;========== Send to LCD, data is in W =====================================================================================

LCD_Ascii
		movwf	 LCD_CHAR_HEXA		; On met W dans LCD_CHAR_HEXA,
		swapf	 LCD_CHAR_HEXA, W 	; swap nibbles
		andlw	 B'00001111'		; mask MS bits
		addlw	 0x30				; add 0x30
		call	 LCD_Char			; display it

		movf	 LCD_CHAR_HEXA, W		; put back LCD_CHAR_HEXA in W
		andlw	 B'00001111'		; mask MS bits
		addlw	 0x30				; add 0x30
		call	 LCD_Char			; display it
		return

LCD_Hexa
		movwf	 LCD_CHAR_HEXA		; Move W in LCD_CHAR_HEXA
		swapf	 LCD_CHAR_HEXA, W		;
		andlw	 0x0F				; Mask most 4 significant bits
		addlw	 6				; add 6
		movwf	 LCD_TEMP			; Move W in LCD_TEMP
		btfsc	 LCD_TEMP,4			; Test if bit 4 is ON
		addlw	 "A"-"9"-1			; add ascii of A less the one of 0
		addlw	 "0" - 6 			; Add ascii code of 0, less 6
		call	 LCD_Char			; Display it

LCD_Single
		movf	 LCD_CHAR_HEXA, W		; put back LCD_CHAR_HEXA in W
		andlw	 0x0F				; Mask most 4 significant bits
		addlw	 6				; add 6
		movwf	 LCD_TEMP			; Move W in LCD_TEMP
		btfsc	 LCD_TEMP,4			; test if bit 4 is ON
		addlw	 "A"-"9"-1			; add ascii of A less the one of 0
		addlw	 "0" - 6 			; Add ascii code of 0, less 6
		call	 LCD_Char			; Display it
		return

;==========  Store 40 characters on line 1 ================================================================================

Lcd_char_1
		call	 LCD_Line1			; move cursor to line 1
Lcd_char_11
		MOVLF	 d'40',ptr_count
		POINT	 Lcd_table
		movf	 ptr_pos,W
		addwf	 TBLPTRL
		clrf	 WREG
		addwfc TBLPTRH
		addwfc TBLPTRU

	   REPEAT_
		tblrd*+				; get the value from TABLE with post increment
		movf	 TABLAT,W
		call	 LCD_Char			; send character to LCD
		decf   ptr_count			; move pointer to next char
	   UNTIL_ .Z.

		return

;========== Store 40 characters on line 2 =================================================================================

Lcd_char_2
		call	 LCD_Line2			; move cursor to line 2
		bra	 Lcd_char_11

;========== Initialisation I/O ports ======================================================================================

InitIO
		clrf	 PORTA			; Initialize  port A
		clrf	 PORTB			; Initialize  port B
		clrf	 PORTC			; Initialize  port C
		clrf	 PORTD			; Initialize  port D
		clrf	 PORTE			; Initialize  port E

		MOVLF	 B'11111111',TRISA	;
		MOVLF	 B'11111111',TRISB	; all in input
		MOVLF	 B'00000000',TRISD	;
		MOVLF	 B'00000000',TRISE	;
		MOVLF	 B'00000111',ADCON1	; Digital I/O
		bsf	 PORTC,RX
		bcf	 INTCON2,RBPU		; activate all PullUp resistors

		bsf	 RCON,IPEN			; enable priority levels on interrupts

		bsf	 IPR1,RCIP			; USART High priority
		bcf	 PIR1,RCIF			; Clear interrupt receiver
		bsf	 PIE1,RCIE			; Enable UART receiver interrupt

		bsf	 INTCON,GIEH		; Enable all interrupts

		bsf	 PIE2,TMR3IE		; Enable interrupt for TMR3
		bcf	 PIR2,TMR3IF		; reset timer interrupt overflow
		bsf	 IPR2,TMR3IP		; TMR3 on high priority

		MOVLF	 d'129',SPBRG		; set speed to 4800 bits/sec (40.000.000/(64(129+1))=4807 bits/sec
		bcf	 TXSTA,SYNC			; set asynchronous mode
		bsf	 RCSTA,SPEN			; enable serial port
		bsf	 RCSTA,CREN			; enable receiver
		return

;==========================================================================================================================

I2C_Init
		clrf	 SSPSTAT
		bsf	 SSPSTAT,SMP		; I2C slew rate control disabled
		bsf	 SSPCON1,SSPM3		; I2C master mode in hardware
		bsf	 SSPCON1,SSPEN		; enable SSP module
		MOVLF	 d'99',SSPADD		; BRG value (100 Kbauds) 4O Mhz / ((99+1)*4) = 100 Khz (24 for 400 Khz)
		bsf	 TRISC,RC3			; I2C SCL pin is input
		bsf	 PORTC,RC3			; (will be controlled by SSP)
		bsf	 TRISC,RC4			; I2C SDA pin is input
		bsf	 PORTC,RC4			; (will be controlled by SSP)
		return

;==========================================================================================================================

I2C_Setup
		bsf	 SSPCON2,SEN		; Send start bit
		btfsc	 SSPCON2,SEN		; Has SEN cleared yet?
		bra	 $-2				; No, loop back to test.
rWakeSlave
		nop
		movff	 I2C_ADDR,SSPBUF		; move I2C address to SSPBUF
		btfsc	 SSPSTAT,R_W		; wait for xmit ended
		bra	 $-2				; no, loop back to test

		btfss	 SSPCON2,ACKSTAT		; was ACK received from slave?
		return				; yes, return to calling routine

		bsf	 SSPCON2,RSEN		; send repeated start bit
		btfsc	 SSPCON2,RSEN		; has repeated start been sent yet?
		bra	 $-2				; no, loop back to test

		bra	 rWakeSlave			; send slave address again

;========== Sends one byte of data to slave, hangs if no ACK ===============================================================

WrDATA
		movff	 I2C_DATA,SSPBUF		; move data to SSPBUF

	   REPEAT_
	   UNTIL_ SSPSTAT,R_W == 0		; wait for end of data transfer

	   REPEAT_
	   UNTIL_ SSPCON2,ACKSTAT == 0	; has slave sent ACK ?

		return

;========== receive one byte from slave ===================================================================================

RdDATA
		bsf	 SSPCON2,RCEN		; enable receive mode

	   REPEAT_
	   UNTIL_ SSPCON2,RCEN == 0		; Wait for receive complete

		movff	 SSPBUF,I2C_DATA		; save data to I2C_DATA
		return

;========== send ACK ======================================================================================================

Ack
		bcf	 SSPCON2,ACKDT		; bit sent will be 0
		bsf	 SSPCON2,ACKEN		; prepare ACK

	   REPEAT_
         UNTIL_ SSPCON2,ACKEN == 0

		return

;========== send NACK =====================================================================================================

Nack
		bsf	 SSPCON2,ACKDT		; bit sent will be 1
		bsf	 SSPCON2,ACKEN		; prepare NACK

	  REPEAT_
	  UNTIL_ SSPCON2,ACKEN == 0

		return

;========== Sends stop bit ================================================================================================

Stop
		bsf	 SSPCON2,PEN		; send stop bit

         REPEAT_
	   UNTIL_ SSPCON2,PEN == 0

		return

;========== Compute percent of total time =================================================================================

Percent
		clrf	Work8
		clrf	Work7
		clrf	Work6
		MOVLF	d'100',Work5		; Divide by 100 the result of multiplication and get percent
		call	divide
		return

;========== Add data into 3 bytes =========================================================================================

Add_data
		movf	 PRODL,W			; Add lo byte
		addwf	 RL
		movf	 PRODH,W			; Add mid-lo byte
		skpnc					; No carry in, so just add
		incfsz PRODH,W			; Add carry in to PRODH
		addwf	 RH				; Add and propagate carry_out
		skpnc
		incf	 RU
		return

;========== CMP 16 bits if < bit 1 ON,  if = 00 bit 0 ON, if > bit 2 ON in WREG ===========================================
; enter with literal in Work6 (High) and Work5 (Low)
; compare with PRODL and PRODH
;==========================================================================================================================

CMP16
    		movf    PRODH,W 		; get high byte of comp value
    		subwf   Work6		; subtract values
    		skpnz				; first check if result is 0
    		bra	  $+8 		; if zero compare low bytes
    		skpnc		 		; else test Carry bit
    		retlw   0x04 		; if total time < PROD, return with b'00000100'
    		retlw   0x02       	; if total time > PROD, return with b'00000010'

    		movf    PRODL,W 		; get low byte of comp value
    		subwf   Work5		; subtract values
    		skpnz		 		; first check if result is 0
    		retlw   0x01 		; if total time is equal, return with b'00000001'
    		skpnc				; if Carry set then WORD > COMP
    		retlw   0x04       	; if total time < PROD, return with b'00000100'
    		retlw   0x02       	; if total time > PROD, return with b'00000010'

;========== 24 BIT DIVIDE =================================================================================================

divide
		clrf   Work1		; Clear remainder
		clrf   Work2
		clrf   Work3
		clrf   Work4
		call   slac			; Purge sign bit

		movlw  D'23'		; Loop counter
		movwf  dec1

dvloop	call   slac			; Shift dividend  msb into remainder
		movf   Work7,W
		subwf  Work3,W
		skpz
		bra    dtstgt
		movf   Work6,W
		subwf  Work2,W
		skpz
		bra    dtstgt
		movf   Work5,W
		subwf  Work1,W
dtstgt	skpc				; Carry set if remainder >= divisor
		bra    dremlt

		movf   Work5,W
		; Subtract divisor from remainder
		subwf  Work1
		movf   Work6,W
		skpc
		incfsz Work6,W
		subwf  Work2
		movf   Work7,W
		skpc
		incfsz Work7,W
		subwf  Work3
		clrc
		bsf	 RL,0			; Set quotient bit

dremlt	decfsz dec1			; Next
		bra	 dvloop
		return

;Shift left REG'S

slac
		rlcf	 RL
		rlcf	 RH
		rlcf	 RU
slc		rlcf	 Work1
		rlcf	 Work2
		rlcf	 Work3
		return

;==========================================================================================================================

Heures_pleines

	db	"Heures pleines : $"

Heures_creuses

	db	"Heures creuses : $"

Satellites

	db	" satellites en vue   $"

Ete
	db 	"  Heure ",0h,"t",0h,"$"

Hiver
	db	"Heure hiver$"

Parametres

	db	"Fin des heures creuses : $"

Pourcent1

	db	"Pourcentage du temps de chauffage$"

Pourcent2

	db    "en heures creuses : $"

Msg1

	db	"APPUYEZ SUR LA TOUCHE ROUGE OU NOIRE$"

Msg2

 	db	"POUR AUGMENTER OU DIMINUER LA VALEUR$"

;==========================================================================================================================

DJU_Table		; The DJU corrected is used as a pointer to this table
			; IE for a DJU of 0x3AD which is round to 0x400, we will use 0x04 as a pointer which says D'0085' mn

			data	D'0000',D'0020',D'0040',D'0060',D'0085',D'0110',D'0135',D'0160',D'0185'
			data	D'0210',D'0235',D'0260',D'0285',D'0310',D'0335',D'0360',D'0385',D'0410'
			data	D'0435',D'0460',D'0480',D'0510',D'0535',D'0560',D'0585',D'0610',D'0635'
			data	D'0660',D'0685',D'0710',D'0740',D'0780',D'0820',D'0860',D'0910',D'0960'
			data	D'1010',D'1060',D'1120',D'1180',D'1240',D'1300',D'1360',D'1420',D'1440'

Degre_table		; this is the tenth of degre which is round at the more accurate deci value
			; address one of the sixteen bytes to get the deci value
			data	0x0100,0x0201,0x0303,0x0404,0x0605,0x0706,0x0808,0x0909

Year_Table
;		      	Jan  Feb  Mar  Apr  may  Jun  Jul  Aug  Sep  Oct  Nov  Dec
;	      	Month	 01   02   03   04   05   06   07   08   09   10   11   12
	      	data	0x31,0x28,0x31,0x30,0x31,0x30,0x31,0x31,0x30,0x31,0x30,0x31

Month_Table

;GMT value       		Jan+1  Fev+1  Mar+?  Avr+2  Mai+2  Jui+2  Jul+2  Aou+2  Sep+2  Oct+?  Nov+1  Dec+1
			data	0x0101,0x0201,0x0380,0x0402,0x0502,0x0602,0x0702,0x0802,0x0902,0x1040,0x1101,0x1201

;========== There is a 28 years cycle to get the same day as a weekday ====================================================
;
;========== data starts at year 2000 up to 2027 then the data for year 2028 is the same as 2000 ===========================

March_Table

;	        2000*  2001   2002   2003   2004*  2005   2006   2007   2008*  2009   2010   2011   2012*  2013
	data   0x2601,0x2500,0x3100,0x3000,0x2801,0x2700,0x2600,0x2500,0x3001,0x2900,0x2800,0x2700,0x2501,0x3100

;		  2014   2015   2016*  2017   2018   2019   2020*  2021   2022   2023   2024*  2025   2026   2027
	data   0x3000,0x2900,0x2701,0x2600,0x2500,0x3100,0x2901,0x2800,0x2700,0x2600,0x3101,0x3000,0x2900,0x2800

October_Table
;	        2000*  2001   2002   2003   2004*  2005   2006   2007   2008*  2009   2010   2011   2012*  2013
	data   0x2901,0x2800,0x2700,0x2600,0x3101,0x3000,0x2900,0x2800,0x2601,0x2500,0x3100,0x3000,0x2801,0x2700

;		  2014   2015   2016*  2017   2018   2019   2020*  2021   2022   2023   2024*  2025   2026   2027
	data   0x2600,0x2500,0x3001,0x2900,0x2800,0x2700,0x2501,0x3100,0x3000,0x2900,0x2701,0x2600,0x2500,0x3100

;==========================================================================================================================

Lcd_table	;table for standard message
		;     "0000000000000000111111111111111122222222"	; first line in hex
		;     "4444444444444444555555555555555566666666"	; second line in hex
		;	"0123456789ABCDEF0123456789ABCDEF01234567"
		;	"XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"
		;									ptr:
		db	"     MISE A L'HEURE DU REGULATEUR       " 	;   0
		db	"        RECHERCHE DE SATELLITES         " 	;  40
		db	"Pas de chauffage la journ",0h,"e             " ;  80
		db	"Pas de chauffage la nuit                "	; 120
		db	"           CHAUFFAGE 24h/24             "	; 160
		db    "                                        "	; 200

MonthTable
		db  "janvier$  "
		db  "f",0h,"vrier$  "		; use ''
		db  "mars$     "
		db  "avril$    "
		db  "mai$      "
		db  "juin$     "
		db  "juillet$  "
		db  "ao",2h,"t$     "		; use ''
		db  "septembre$"
		db  "octobre$  "
		db  "novembre$ "
		db  "d",0h,"cembre$ "		; use ''

;===============================================================================

		END        			; directive 'end of program'


