;*******************************************************************************
;                                                                              *
;   Filename:       Main.asm                    Projet: Amplificateur_4520v1_1 *
;   Author:         Philippe DOUILLARD                                         *
;                                                                              *
;   Commentaire:    Module principal du projet                                 *
;                   Control des ventillateurs en PWM et PID                    *
;                   Sondes temprature sur entrees ANA                         *
;                   Tempos sur Timer matriel                                  *
;                                                                              *
;   Version: 1.0    07/04/2013      Version Initiale du projet                 *
;   Version: 1.1    04/02/2014      PWM, PID, Timers...                        *
;                                                                              *
;*******************************************************************************
;                                                                              *
;    Files required:    P18F4520.INC                                           *
;                                                                              *
;*********************** DIRECTIVE D'ASSEMBLAGE ********************************

    LIST P=18F4520, F=INHX32    ;directive to define processor and file format
    #include <P18F4520.INC>     ;processor specific variable definitions

;************************* CONFIGURATION BITS **********************************

    CONFIG      OSC=HS          ;Oscillator Selection bits
    CONFIG      FCMEN = OFF     ;Fail-Safe Clock Monitor Enable bit
    CONFIG      IESO = OFF      ;Internal/External Oscillator Switchover bit

    CONFIG      PWRT = ON       ;Power-up Timer Enable bit
    CONFIG      BOREN = OFF     ;Brown-out Reset Enable bits
    CONFIG      BORV = 3        ;Brown Out Reset Voltage bits
    CONFIG      WDT = OFF       ;Watchdog Timer Enable bit
    CONFIG      WDTPS = 128     ;Watchdog Timer Postscale Select bits
    CONFIG      CCP2MX = PORTC  ;CCP2 pin Mux enabled (PORTC/PORBE)
    CONFIG      PBADEN = ON     ;PORTB A/D Enable bit
    CONFIG      LPT1OSC = OFF   ;Low-Power Timer1 Oscillator Enable bit
    CONFIG      MCLRE = ON      ;MCLR Pin Enable bit
    CONFIG      STVREN = ON     ;Stack Full/Underflow Reset Enable bit
    CONFIG      LVP = OFF       ;Single-Supply ICSP Enable bit
    CONFIG      XINST = ON      ;Extended Instruction Set Enable bit
    CONFIG      DEBUG = ON      ;Background Debugger Enable bit

    CONFIG      CP0 = OFF       ;Code Protection bit
    CONFIG      CP1 = OFF       ;Code Protection bit
    CONFIG      CP2 = OFF       ;Code Protection bit
    CONFIG      CP3 = OFF       ;Code Protection bit
    CONFIG      CPB = OFF       ;Boot Block Code Protection bit
    CONFIG      CPD = OFF       ;Data EEPROM Code Protection bit
    CONFIG      WRT0 = OFF      ;Write Protection bit
    CONFIG      WRT1 = OFF      ;Write Protection bit
    CONFIG      WRT2 = OFF      ;Write Protection bit
    CONFIG      WRTC = OFF      ;Configuration Register Write Protection bit
    CONFIG      WRTB = OFF      ;Boot Block Write Protection bit
    CONFIG      WRTD = OFF      ;Data EEPROM Write Protection bit
    CONFIG      EBTR0 = OFF     ;Table Read Protection bit
    CONFIG      EBTR1 = OFF     ;Table Read Protection bit
    CONFIG      EBTR2 = OFF     ;Table Read Protection bit
    CONFIG      EBTR3 = OFF     ;Table Read Protection bit
    CONFIG      EBTRB = OFF     ;Boot Block Table Read Protection bit

;****************** VARIABLES DECLARATION GLOBALES *****************************

    global  temp1, temp2, temp3,temp4,temp5,temp6,temp7
    global  memo1, memo2,memo3,memo4

    global  THERMO1_LOW,THERMO1_HIGH,THERMO2_LOW,THERMO2_HIGH
    global  PWM1_LOW,PWM1_HIGH,PWM1_MIN_LOW,PWM1_MIN_HIGH
    global  PWM1_MAX_LOW,PWM1_MAX_HIGH,PWM2_LOW,PWM2_HIGH,PWM2_MIN_LOW

    global  cycleL,cycleH,COMPT_ISR,PWM_LED
    global  CPT_REBOND_1,CPT_REBOND_2,CPT_REBOND_3

    global Main

;************************ MODULES EXTERNES *************************************
    
    extern  Delay_short,Delay_long,Delay_500ms,Delay_100ms,Bouton_Poussoir

;*************** VARIABLE RESERVATION (RAM) ****** GROUPE 0 ****** 127 Bytes ***

gpr0            UDATA

WREG_TEMP       RES 1   ; variable in RAM for context saving
STATUS_TEMP     RES 1   ; variable in RAM for context saving
BSR_TEMP        RES 1   ; variable in RAM for context saving

temp1           res 1   ; Variable temporaire 1
temp2           res 1   ; Variable temporaire 2
temp3           res 1   ; Variable temporaire 3
temp4           res 1   ; Variable temporaire 4
temp5           res 1   ; Variable temporaire 5
temp6           res 1   ; Variable temporaire 6
temp7           res 1   ; Variable temporaire 7

memo1           res 1   ; Memo bit (voir quivalences)
memo2           res 1   ; Memo bit (voir quivalences)
memo3           res 1   ; Memo bit (voir quivalences)
memo4           res 1   ; Memo bit (voir quivalences)

cycleL          res 1   ; Variable comptage cycle Low  (gestion priorites)
cycleH          res 1   ; Variable comptage cycle High (gestion priorites)
COMPT_BP_L      res 1   ; Compteur temporisation BP ON/OFF LOW
COMPT_BP_M      res 1   ; Compteur temporisation BP ON/OFF HIGH
COMPT_BP_H      res 1   ; Compteur temporisation BP ON/OFF HIGH
COMPT_CT_L      res 1   ; Compteur temporisation Contact EXT LOW
COMPT_CT_M      res 1   ; Compteur temporisation Contact EXT HIGH
COMPT_CT_H      res 1   ; Compteur temporisation Contact EXT HIGH
COMPT_ISR       res 1   ; Compteur Interuption
PWM_LED         res 1   ; Valeur 0-255 PWM intensit LED
PWM_LED_MAX     res 1   ; Valeur Max PWM intensit LED

CPT_REBOND_1    res 1   ; Compteur rebond 1 (Contact remote)
CPT_REBOND_2    res 1   ; Compteur rebond 2 (Seuil TEMPERATURE 1)
CPT_REBOND_3    res 1   ; Compteur rebond 3 (Seuil TEMPERATURE 2)
CPT_PAUSE       res 1   ; Compteur PAUSE en fonction ampli

;**************** VARIABLE RESERVATION (RAM) ****** GROUPE 1 ****** 127 Bytes ***

gpr1            UDATA

THERMO1_LOW     res 1   ; Sonde Thermoresistance 1 LOW
THERMO1_HIGH    res 1   ; Sonde Thermoresistance 1 HIGH
THERMO2_LOW     res 1   ; Sonde Thermoresistance 2 LOW
THERMO2_HIGH    res 1   ; Sonde Thermoresistance 2 HIGH

PWM1_LOW        res 1   ; Valeur PWM Low    Sortie Ventilateur 1
PWM1_HIGH       res 1   ; Valeur PWM High   Sortie Ventilateur 1
PWM1_MIN_LOW    res 1   ; Valeur PWM Low    Limite Minimale 1
PWM1_MIN_HIGH   res 1   ; Valeur PWM High   Limite Minimale 1
PWM1_MAX_LOW    res 1   ; Valeur PWM Low    Limite Maximale 1
PWM1_MAX_HIGH   res 1   ; Valeur PWM High   Limite Maximale 1

PWM2_LOW        res 1   ; Valeur PWM Low    Sortie Ventilateur 2
PWM2_HIGH       res 1   ; Valeur PWM High   Sortie Ventilateur 2
PWM2_MIN_LOW    res 1   ; Valeur PWM Low    Limite Minimale 2
PWM2_MIN_HIGH   res 1   ; Valeur PWM High   Limite Minimale 2
PWM2_MAX_LOW    res 1   ; Valeur PWM Low    Limite Maximale 2
PWM2_MAX_HIGH   res 1   ; Valeur PWM High   Limite Maximale 2

;******************* VARIABLE RESERVATION (ACCES RAM) **************************

                UDATA_ACS

exemple         res 1   ; exemple de variable en acces ram

;*************************** EEPROM DATA RESERVATION ***************************

DATA_EEPROM     CODE	0xf00000

                DE	"Test Data",0,1,2,3,4,5

;*********************** VARIABLES DEFINITION **********************************

#define     DEF_CC1         memo1,0     ; bit0	Defaut CC1
#define     DEF_CC2         memo1,1     ; bit1	Defaut CC2
#define     DEF_INT1        memo1,2     ; bit2	Defaut INT1
#define     DEF_INT2        memo1,3     ; bit3	Defaut INT2
#define     DEF_TEMP1       memo1,4     ; bit4  Defaut TEMP1
#define     DEF_TEMP2       memo1,5     ; bit5  Defaut TEMP2
#define     SEUIL_TEMP1     memo1,6     ; bit6  Seuil  TEMP1
#define     SEUIL_TEMP2     memo1,7     ; bit7  Seuil  TEMP2

#define     FLAG_Invers     memo2,0     ; bit0  PWM LED - flag inv clignotement
#define     FLAG_FlashON    memo2,1     ; bit1  PWM LED - flag Cligno ON/OFF
#define     FLAG_Flash50    memo2,2     ; bit2  Bit Clignotant 50/50
#define     FLAG_Flash10    memo2,3     ; bit3  Clignotant 10/90
#define     SEL_AMP1        memo2,4     ; bit4  Selection Prog 1 (SINGLOTRON 1)
#define     SEL_AMP2        memo2,5     ; bit5  Selection Prog 2 (SINGLOTRON 2)
#define     SEL_AMP3        memo2,6     ; bit6  Selection Prog 3 (MOSQUITO)
#define     SEL_AMP4        memo2,7     ; bit7  Selection Prog 4 (ICE POWER)

#define     FLAG_Prechauf1  memo3,0     ; bit0  Seuil Prchauffage 1
#define     FLAG_Prechauf2  memo3,1     ; bit1  Seuil Prchauffage 2
#define     FLAG_MUTE1      memo3,2     ; bit2  Memo  Mute1
#define     FLAG_MUTE2      memo3,3     ; bit3  Memo  Mute2
#define     FLAG_PH_ARRET   memo3,4     ; bit4  Phase Arret
                                        ; bit5
                                        ; bit6
                                        ; bit7

#define     FLAG_REBOND_1   memo4,0     ; bit0  Etat bascule Anti rebond 1
#define     FLAG_ANT_1      memo4,1     ; bit1  Etat anterieur front montant 1
#define     FLAG_FM_1       memo4,2     ; bit2  Etat front montant 1
#define     FLAG_FD_1       memo4,3     ; bit3  Etat front descendant 1
#define     FLAG_REBOND_2   memo4,4     ; bit0  Etat bascule Anti rebond 2
#define     FLAG_REBOND_3   memo4,5     ; bit0  Etat bascule Anti rebond 3
                                        ; bit6
                                        ; bit7

;**************************** DEFINITION I/O ***********************************

;                       PORTA,0     Analogue input      Sonde Thermistance 1
;                       PORTA,1     Analogue input      Sonde Thermistance 2
#define     CT_CC1      PORTA,2   ; digital input       Defaut CC 1
#define     CT_CC2      PORTA,3   ; digital input       Defaut CC 2
#define     CT_INT1     PORTA,4   ; digital input       Defaut INTERNE 1
#define     CT_INT2     PORTA,5   ; digital input       Defaut INTERNE 2
;                       PORTA,6     (OSC1)              Rserv
;                       PORTA,7     (OSC2)              Rserv
;
#define     LED_CC1     PORTB,0   ; digital output      LED Defaut CC  1
#define     LED_CC2     PORTB,1   ; digital output      LED Defaut CC  2
#define     LED_INT1    PORTB,2   ; digital output      LED Defaut INT 1
#define     LED_INT2    PORTB,3   ; digital output      LED Defaut INT 2
;                       PORTB,4     digital output      -
#define     RELAIS      PORTB,5   ; digital output      RELAIS Mise sous Tension
;                       PORTB,6     (PGC)               Rserv MPLAB
;                       PORTB,7     (PGD)               Rserv MPLAB

#define     LED_BLEU    PORTC,0   ; digital output      LED BLEU
;                       PORTC,1     digital output      Ventillateur 1
;                       PORTC,2     digital output      Ventillateur 2
;                       PORTC,3     serial SPI          SPI
;                       PORTC,4     serial SPI          SPI
;                       PORTC,5     serial SPI          SPI
;                       PORTC,6     digital output      -
;                       PORTC,7     digital output      -
;
#define     LED_TEMP1   PORTD,0   ; digital output      AMP - LED Defaut TEMP  1
#define     LED_TEMP2   PORTD,1   ; digital output      AMP - LED Defaut TEMP  1
#define     MUTE1       PORTD,2   ; digital output      AMP - MUTE  1
#define     MUTE2       PORTD,3	  ; digital output      AMP - MUTE  2
;                       PORTD,4     digital output      -
#define     SEL1        PORTD,5   ; digital input       Selecteur 1
#define     SEL2        PORTD,6   ; digital input       Selecteur 2
#define     SEL3        PORTD,7   ; digital input       Selecteur 3

#define     BP_ONOFF    PORTE,0   ; digital Input       BP Marche/Arret
#define     CT_ONOFF    PORTE,1   ; digital Input       Contact relais Externe
;                       PORTE,2     digital Input       -

;*********************** CONSTANTES DEFINITION *********************************

SP_CHAUF_L     EQU     d'130'       ; Seuil Tc     Arret prchauffage
SP_CHAUF_H     EQU     0x01         ; Seuil Tc     Arret prchauffage
SP_REGUL_L     EQU     d'157'       ; Consigne Tc  Rgulation
SP_REGUL_H     EQU     0x01         ; Consigne Tc  Rgulation
SP_DF_TP_L     EQU     d'182'       ; Seuil Defaut  Tc
SP_DF_TP_H     EQU     0x01         ; Seuil Defaut  Tc

;************************** RESET VECTOR ***************************************
RESET_VECTOR	CODE	0x0000
                goto	Main		;go to start of main code
;****************** HIGHT PRIORITY INTERRUPT VECTOR ****************************
HI_INT_VECTOR	CODE	0x0008
                goto	HighInt		;go to high priority interrupt routine
;******************** LOW PRIORITY INTERRUPT VECTOR ****************************
LOW_INT_VECTOR	CODE	0x0018
                goto	LowInt		;go to low priority interrupt routine
;******************* HIGHT PRIORITY INTERRUPT ROUTINE **************************

                CODE
HighInt:
                movff	STATUS,STATUS_TEMP	; Save STATUS register
                movff	WREG,WREG_TEMP		; Save working register
                movff	BSR,BSR_TEMP		; Save BSR register

                btfss   PIR1, TMR1IF        ; TMR1 Interuption?
                bra     HighInt_suite

                bcf     T1CON,TMR1ON        ; turn OFF Timer1
                bcf     PIR1, TMR1IF        ; Clear TMR1 Interuption Flag

                incf    COMPT_ISR,f         ; Incremente compteur
                movlw   0xFF                ; Preload TMR1      (TIMER1)
                movwf   TMR1H               ; TMR1 HIGH
                movlw   0x00
                movwf   TMR1L               ; TMR1 LOW

                movlw   0xFF                ; Preload TMR3      (TIMER3)
                movwf   TMR3H               ; TMR3 HIGH
                MOVF    PWM_LED,W
                movwf   TMR3L               ; TMR3 LOW

                btfss   FLAG_PH_ARRET       ; Si phase d'arret saut LED BLEU
                bsf     LED_BLEU            ; LED Bleu BP
                bsf     T1CON,TMR1ON        ; turn ON Timer1
                bsf     T3CON,TMR3ON        ; turn on Timer3
                
HighInt_suite   btfss   PIR2, TMR3IF        ; TMR3 Interuption?
                bra     HighInt_Exit

                bcf     T3CON,TMR3ON        ; turn OFF Timer3
                bcf     PIR2, TMR3IF        ; Clear TMR3 Interuption Flag
                btfss   FLAG_PH_ARRET       ; Si phase d'arret saut LED BLEU
                bcf     LED_BLEU            ; LED Bleu BP

HighInt_Exit
                movff	BSR_TEMP,BSR		; Restore BSR register
                movff	WREG_TEMP,WREG		; Restore working register
                movff	STATUS_TEMP,STATUS	; Restore STATUS register
                retfie	FAST

;******************** LOW PRIORITY INTERRUPT ROUTINE ***************************

LowInt:
                movff	STATUS,STATUS_TEMP	;save STATUS register
                movff	WREG,WREG_TEMP		;save working register
                movff	BSR,BSR_TEMP		;save BSR register

                ; Programme here

                movff	BSR_TEMP,BSR		;restore BSR register
                movff	WREG_TEMP,WREG		;restore working register
                movff	STATUS_TEMP,STATUS	;restore STATUS register
                retfie


;*******************************************************************************
;********************** PROGRAMME PRINCIPALE ***********************************
;*******************************************************************************
Main:
;------------------------ CONFIG PORTS I/O -------------------------------------

		clrf    memo4
RESTART		clrf    PORTA          ; RAZ PORTA
		clrf    PORTB          ; RAZ PORTB
		clrf    PORTC          ; RAZ PORTC
		clrf    PORTD          ; RAZ PORTD
		clrf    PORTE          ; RAZ PORTE

		MOVLW   B'00001101'	; 0000 1101
		MOVWF   ADCON1		; RA0 et RA1 ANA Input RA2:5 Digital Input
		MOVLW   B'10101100'	; Right justified - 12 TAD - FOSC/8
		MOVWF   ADCON2		;

		MOVLW   B'00000111'	; 0000 0111
		MOVWF   CMCON         ; Comparateur OFF (RA0:2 digital)
		MOVLW   0xFF          ; 1111 1111
		MOVWF   TRISA         ; RA0:5 Inputs

		clrf    TRISB          ; RB0:7 digital output

		movlw   B'00010000'	; tout en sortie (SDO,SCK) sauf (SDI entre)
		movwf   TRISC         ; Direction port C

		movlw   B'11000000'   ; 0-5 Input 6 et 7 Output
		movwf   TRISD         ; RD

 		movlw   B'00000111'   ; 0000 0000
		movwf   TRISE         ; RE 0:2 Inputs

;----------------------- CONFIG PWM 2 -------------------------------------------
        
        bsf     TRISC,1             ; Disable PWM OUT
        clrf	CCP2CON             ; CCP module is off
        bsf     CCP2CON, CCP2M3		; select PWM mode
        bsf     CCP2CON, CCP2M2		; select PWM mode
        movlw	0x3F                ; Set PWM frequency to 78.12kHz
        movwf	PR2                 ;
        bcf     TRISC, 1            ;make channel 1 an output
        movlw	0x00
        movwf	CCPR2L

        movlw   b'00000000'         ; Postsacaler 1:1 Timer OFF Prescaler 1:1
        movwf	T2CON               ;clear T2CON
        clrf	TMR2                ;clear Timer2
        bsf     T2CON,TMR2ON        ;turn on Timer2

;----------------------- CONFIG PWM 1 -------------------------------------------

        bsf     TRISC,2             ; Disable PWM OUT
        clrf	CCP1CON             ; CCP module is off
        bsf     CCP1CON, CCP1M3		; select PWM mode
        bsf     CCP1CON, CCP1M2		; select PWM mode
        bcf     TRISC, 2            ;make channel 2 an output
        movlw	0x00
        movwf	CCPR1L

;--------------------- INIT PORTS I/O ------------------------------------------

        clrf    PORTA
        clrf    PORTB
        clrf    PORTC
        clrf    PORTD
        clrf    PORTE

;--------------------- INIT Variables ------------------------------------------

		clrf    temp1               ; Reset des variables
		clrf    temp2
		clrf    temp3
		clrf    memo1
        clrf    memo2
        clrf    memo3
        ;clrf    memo4
        clrf    cycleL
        movlw   d'240'              ; Valeur Cycle HIHT pour LOW PRIORITY
        movwf   cycleH              ; Valeur au dmarrage

        clrf    THERMO1_LOW
        clrf    THERMO1_HIGH
        clrf    THERMO2_LOW
        clrf    THERMO2_HIGH
        clrf    COMPT_BP_L          ; RAZ compteur
        clrf    COMPT_BP_M          ; RAZ compteur
        clrf    COMPT_BP_H          ; RAZ compteur
        clrf    COMPT_CT_L          ; RAZ compteur
        clrf    COMPT_CT_M          ; RAZ compteur
        clrf    COMPT_CT_H          ; RAZ compteur

        bcf     DEF_INT1            ; RESET Memo Defaut INT 1
        bcf     DEF_INT2            ; RESET Memo Defaut INT 2
        bcf     DEF_CC1             ; RESET Memo Defaut CC 1
        bcf     DEF_CC2             ; RESET Memo Defaut CC 2
        bcf     DEF_TEMP1           ; RESET Memo Defaut TEMP 1
        bcf     DEF_TEMP2           ; RESET Memo Defaut TEMP 2

        MOVLW   0x05
        movwf   PWM_LED

        MOVLW   0x01
        movwf   PWM_LED_MAX

        bsf     FLAG_Prechauf1          ; Prechauffage 1 ON
        bsf     FLAG_Prechauf2          ; Prechauffage 2 ON

        btfsc   SEL3                    ; SEL3=ICE
        bcf     FLAG_Prechauf1          ; Prechauffage 1 OFF
        btfsc   SEL3                    ; SEL3=ICE
        bcf     FLAG_Prechauf2          ; Prechauffage 2 OFF

        movlw   d'01'                   ; W=0 -> SINGLOTRONB 1
        btfsc   SEL1                    ; SEL1 = SINGLOTRON 2
        movlw   d'02'                   ; W=1 -> SINGLOTRONB 2
        btfsc   SEL2                    ; SEL1 = ZENQUITO
        movlw   d'03'                   ; W=1 -> ZENQUITO
        btfsc   SEL3                    ; SEL1 = ICEPOWER
        movlw   d'04'                   ; W=1 -> ICEPOWER
        movwf   CPT_PAUSE               ; CPT_PAUSE=W

        MOVLW   d'18'
        MOVWF   PWM1_MIN_LOW
        MOVLW   d'00'
        MOVWF   PWM1_MIN_HIGH

        MOVLW   b'11111111'         
;       MOVLW   b'00011111'         MODIF POUR ZENQUITO
        MOVWF   PWM1_MAX_LOW
        MOVLW   b'00000000'
        MOVWF   PWM1_MAX_HIGH

        MOVFF   PWM1_MIN_LOW,PWM1_LOW
        MOVFF   PWM1_MIN_HIGH,PWM1_HIGH

        MOVLW   d'18'
        MOVWF   PWM2_MIN_LOW
        MOVLW   d'00'
        MOVWF   PWM2_MIN_HIGH

        MOVLW   b'11111111'         
;       MOVLW   b'00011111'         MODIF POUR ZENQUITO
        MOVWF   PWM2_MAX_LOW
        MOVLW   b'0000000'
        MOVWF   PWM2_MAX_HIGH

        MOVFF   PWM2_MIN_LOW,PWM2_LOW
        MOVFF   PWM2_MIN_HIGH,PWM2_HIGH



;*******************************************************************************
;*************************** PHASE DEMARRAGE ***********************************
;*******************************************************************************
PHASE_DEMARRAGE

                call    Bouton_Poussoir

;---------------------- DETECTION PHASE DEMARRAGE -------------- BP ON/OFF -----

                btfss   BP_ONOFF            ; Test si appuie BP
                bra     MARCHE_OFF_BP
MARCHE_ON_BP    incf    COMPT_BP_L ,F       ; Add one to LSB
                btfsc   STATUS,Z            ; Overflowed?
                incf    COMPT_BP_M,F        ; Yes, add one to MSB
                btfsc   STATUS,Z            ; Overflowed?
                incf    COMPT_BP_H,F        ; Yes, add one to MSB
                bra     TEST_MARCHE_BP
MARCHE_OFF_BP   clrf    COMPT_BP_L          ; RAZ compteur
                clrf    COMPT_BP_H          ; RAZ compteur
TEST_MARCHE_BP  movlw   0x01                ; Valeur Compteur
                cpfsgt  COMPT_BP_H          ; Compteur > Valeur
                bra     TEST_MARCHE_BP_suite
                bra     PHASE_MARCHE         ; Saute phase d'arret
TEST_MARCHE_BP_suite

;----------------------- DETECTION PHASE DEMARRAGE ------------- CT EXTERNE ----

                btfss   FLAG_FM_1           ; Test Front Montant Contact REMOTE
                bra     PHASE_DEMARRAGE
                bra     PHASE_MARCHE         ; Saute phase d'arret


;******************************* MARCHE ****************************************
PHASE_MARCHE
                clrf    COMPT_BP_L
                clrf    COMPT_BP_M
                clrf    COMPT_BP_H
                clrf    COMPT_CT_L
                clrf    COMPT_CT_M
                clrf    COMPT_CT_H

;--------------------- Configuration Tmers et interuptions ---------------------

        CLRF    TMR1H
        CLRF    TMR1L
        MOVLW   b'00110000'     ; 2x8bits TMR1OSC:OFF Pres1:8 IntClock:FOSC/4
        MOVWF   T1CON
        MOVLW   0xFF            ; Preload TMR1      (TIMER1)
        MOVWF   TMR1H           ; TMR1 HIGH
        MOVLW   0x00
        MOVWF   TMR1L           ; TMR1 LOW

        CLRF    TMR3H
        CLRF    TMR3L
        MOVLW   b'00110000'     ; 2x8bits TMR1OSC:OFF Pres1:8 IntClock:FOSC/4
        MOVWF   T3CON
        MOVLW   0xFF            ; Preload TMR3      (TIMER3)
        MOVWF   TMR3H           ; TMR3 HIGH
        MOVF    PWM_LED,W
        MOVWF   TMR3L           ; TMR3 LOW

        bsf     RCON,IPEN       ;enable priority interrupts.
        bsf     IPR1,TMR1IP     ;set Timer1 as a high priority interrupt source
        bsf     IPR2,TMR3IP     ;set Timer3 as a high priority interrupt source
        bcf     PIR1,TMR1IF     ;clear the Timer1 interrupt flag
        bcf     PIR2,TMR3IF     ;clear the Timer3 interrupt flag
        bsf     PIE1,TMR1IE     ;enable Timer1 interrupts
        bsf     PIE2,TMR3IE     ;enable Timer3 interrupts
        bsf     INTCON,GIEH     ; Set the global High interrupt enable bits
        bcf     INTCON,GIEL     ; Set the global Low interrupt enable bits

        bsf     T1CON,TMR1ON	; turn on Timer1
        bsf     T3CON,TMR3ON	; turn on Timer3

;----------------------------- VITESSE PWM -------------------------------------

        MOVLW   0xFF            ; PWM 1  100%
        MOVWF   CCPR2L          ; SET high 8 bits of PWM duty cycle
        bsf     CCP2CON,4       ; SET 2 bits of PWM Duty cycle DC1B
        bsf     CCP2CON,5       ; SET 2 bits of PWM Duty cycle DC1B

        MOVLW   0xFF            ; PWM 2  100%
        MOVWF   CCPR1L          ; SET high 8 bits of PWM duty cycle
        bsf     CCP1CON,4       ; SET 2 bits of PWM Duty cycle DC1B
        bsf     CCP1CON,5       ; SET 2 bits of PWM Duty cycle DC1B

;------------------------ Squence LEDS + RELAIS -------------------------------

        bsf     RELAIS          ; Relais MES
        call    Delay_500ms
        call    Delay_500ms

        bsf     LED_CC1         ; LED Defaut CC1
        call    Delay_100ms

        bcf     LED_CC1         ; LED Defaut CC1
        bsf     LED_INT1        ; LED Defaut INT1
        call    Delay_100ms

        bcf     LED_INT1        ; LED Defaut INT1
        bsf     LED_TEMP1       ; LED Defaut Temp1
        call    Delay_100ms

        bcf     LED_TEMP1       ; LED Defaut Temp1
        bsf     LED_TEMP2       ; LED Defaut Temp2
        call    Delay_100ms

        bcf     LED_TEMP2       ; LED Defaut Temp2
        bsf     LED_INT2        ; LED Defaut INT2
        call    Delay_100ms

        bcf     LED_INT2        ; LED Defaut INT2
        bsf     LED_CC2         ; LED Defaut CC2
        call    Delay_100ms

        bcf     LED_CC2         ; LED Defaut CC2
        call    Delay_100ms

;----------------------------- VITESSE PWM -------------------------------------

        MOVLW   0xFF            ; PWM 1 au minimum
        MOVWF   CCPR2L          ; SET high 8 bits of PWM duty cycle
        bcf     CCP2CON,4       ; SET 2 bits of PWM Duty cycle DC1B
        bcf     CCP2CON,5       ; SET 2 bits of PWM Duty cycle DC1B

        MOVLW   0xFF            ; PWM 2 au minimum
        MOVWF   CCPR1L          ; SET high 8 bits of PWM duty cycle
        bcf     CCP1CON,4       ; SET 2 bits of PWM Duty cycle DC1B
        bcf     CCP1CON,5       ; SET 2 bits of PWM Duty cycle DC1B


;------------------------------ Tempo PAUSE en fonction AMPLI ------------------
        movff   CPT_PAUSE,temp1
PAUSE_loop
        call    Delay_500ms
        call    Delay_500ms
        decfsz  temp1            ; dcrmenter compteur pause
        bra     PAUSE_loop
;------------------------------ UNMUTE -----------------------------------------
        bsf     MUTE1           ; UN-MUTE 1
        call    Delay_500ms
        bsf     MUTE2           ; UN-MUTE 2

        call    Delay_500ms

        goto    High_Priority   ; Dmarrage programme principal

;*******************************************************************************
;****************************** PHASE ARRET ************************************
;*******************************************************************************
PHASE_ARRET
        bsf     FLAG_PH_ARRET   ; SET Phase d'arret

;------------------------------ Tempo PAUSE en fonction AMPLI ------------------
        movff   CPT_PAUSE,temp1
PAUSE_loop2
        call    Delay_500ms
        call    Delay_500ms
        decfsz  temp1            ; dcrmenter compteur pause
        bra     PAUSE_loop2

        bcf     MUTE1
        call    Delay_100ms
        bcf     MUTE2
        call    Delay_100ms
        call    Delay_100ms
        bcf     DEF_INT1        ; RESET Memo Defaut INT 1
        bcf     DEF_INT2        ; RESET Memo Defaut INT 2
        bcf     DEF_CC1         ; RESET Memo Defaut CC 1
        bcf     DEF_CC2         ; RESET Memo Defaut CC 2
        bcf     DEF_TEMP1       ; RESET Memo Defaut TEMP 1
        bcf     DEF_TEMP2       ; RESET Memo Defaut TEMP 2

        bcf     LED_BLEU
        bcf     RELAIS
        bcf     LED_CC1         ; LED Defaut CC1
        bcf     LED_CC2         ; LED Defaut CC2
        bcf     LED_INT1        ; LED Defaut INT1
        bcf     LED_INT2        ; LED Defaut INT2
        bcf     LED_TEMP1       ; LED Defaut Temp1
        bcf     LED_TEMP2       ; LED Defaut Temp2

        clrf    COMPT_BP_L
        clrf    COMPT_BP_H
        clrf    COMPT_CT_L
        clrf    COMPT_CT_H
        bcf     T1CON,TMR1ON	; turn OFF Timer1
        bcf     T3CON,TMR3ON	; turn OFF Timer3
        bcf     memo4,3         ; RESET bit front decendant Contact REMOTE

        bcf     CCP2CON,4       ; SET 2 bits of PWM Duty cycle DC1B
        bcf     CCP2CON,5       ; SET 2 bits of PWM Duty cycle DC1B
        bcf     CCP1CON,4       ; SET 2 bits of PWM Duty cycle DC1B
        bcf     CCP1CON,5       ; SET 2 bits of PWM Duty cycle DC1B

        MOVLW   0x64            ; Nombre de cycle
        MOVWF   temp3           ; Charge compteur boucle

ARRET_BCK1

        MOVFF   temp3,CCPR2L    ; SET high 8 bits of PWM 1 duty cycle
        MOVFF   temp3,CCPR1L    ; SET high 8 bits of PWM 2 duty cycle

        CALL    Delay_500ms     ; SUBROUTINE delay
        CALL    Delay_500ms     ; SUBROUTINE delay
        bsf     LED_BLEU        ; SET LED
        CALL    Delay_long      ; SUBROUTINE delay
        bcf     LED_BLEU        ; RESET LED

        decf    temp3           ; dcrmenter compteur boucle
        movlw   0x04            ; Valeur Compteur
        cpfslt  temp3           ; Compteur > Valeur
        GOTO    ARRET_BCK1

        MOVLW   0x00            ; PWM 1 0%
        MOVWF   CCPR2L          ; SET high 8 bits of PWM duty cycle
        bcf     CCP2CON,4       ; SET 2 bits of PWM Duty cycle DC1B
        bcf     CCP2CON,5       ; SET 2 bits of PWM Duty cycle DC1B

        MOVLW   0x00            ; PWM 2 0%
        MOVWF   CCPR1L          ; SET high 8 bits of PWM duty cycle
        bcf     CCP1CON,4       ; SET 2 bits of PWM Duty cycle DC1B
        bcf     CCP1CON,5       ; SET 2 bits of PWM Duty cycle DC1B

        goto    RESTART

;*******************************************************************************
;*********************** BOUCLE PRINCIPAL ******************* HIGH PRIORITY ****
;*******************************************************************************

High_Priority

        incf    cycleL,f
        btfsc   STATUS,Z      ; Overflowed?
        incf    cycleH,f      ; Yes, add one to MSB

        call    Bouton_Poussoir

;--------------------------- ANALOGIQUE ----------------------------- AN0 ------

        MOVLW   B'11000001'     ; Select channel AN0
        MOVWF   ADCON0          ; Turn ADC On

        CALL    Delay_short     ; Acquisiton delay

        BSF     ADCON0,GO       ; Start conversion
back    BTFSC   ADCON0,GO       ; Is conversion done?
        GOTO    back            ; No, test again

        MOVF    ADRESH,W        ; Read upper 2 bits
        MOVWF   THERMO1_HIGH      ; Store in variable High

        MOVF    ADRESL,W        ; Read lower 8 bits
        MOVWF   THERMO1_LOW     ; Store in variable Low

;--------------------------- ANALOGIQUE ----------------------------- AN1 ------

        MOVLW   B'11000101'     ; Select channel AN1
        MOVWF   ADCON0          ; Turn ADC On

        CALL    Delay_short     ; Acquisiton delay

        BSF     ADCON0,GO       ; Start conversion
back2   BTFSC   ADCON0,GO       ; Is conversion done?
        GOTO    back2            ; No, test again

        MOVF    ADRESH,W        ; Read upper 2 bits
        MOVWF   THERMO2_HIGH      ; Store in variable High

        MOVF    ADRESL,W        ; Read lower 8 bits
        MOVWF   THERMO2_LOW     ; Store in variable Low

;---------------------------- DEFAUTS ------------------------------- CC1 ------

        btfsc   SEL3            ; Si ICEPOWER alors pas defaut CC
        bra     jump_CC
        btfsc   CT_CC1          ; Test si pas Contact CC 1
        bsf     DEF_CC1         ; SET Memo Defaut CC 1 (bsf)
        btfsc   BP_ONOFF        ; RESET Defaut?  (Appui sur BP ON/OFF)
        bcf     DEF_CC1         ; RESET Memo Defaut CC 1

;---------------------------- DEFAUTS ------------------------------- CC2 ------
        btfsc   CT_CC2          ; Test si pas Contact CC 2
        bsf     DEF_CC2         ; SET Memo Defaut CC 2 (bsf)
        btfsc   BP_ONOFF        ; RESET Defaut?  (Appui sur BP ON/OFF)
        bcf     DEF_CC2         ; RESET Memo Defaut CC 2

;---------------------------- DEFAUTS ------------------------------ INT 1 -----
jump_CC btfss   SEL3            ; Si ICEPOWER alors defaut CC sinon saute
        bra     jump_INT

        btfsc   CT_INT1         ; Test si pas Contact INT 1
        bsf     DEF_INT1        ; SET Memo Defaut INT 1
        btfsc   BP_ONOFF        ; RESET Defaut?  (Appui sur BP ON/OFF)
        bcf     DEF_INT1        ; RESET Memo Defaut INT 1

;---------------------------- DEFAUTS ------------------------------ INT 2 -----
        btfsc   CT_INT2         ; Test si pas Contact INT 2
        bsf     DEF_INT2        ; SET Memo Defaut INT 2
        btfsc   BP_ONOFF        ; RESET Defaut?  (Appui sur BP ON/OFF)
        bcf     DEF_INT2        ; RESET Memo Defaut INT 2

;------------------------------ SEUIL ------------------------ TEMPERATURE 1 ---

jump_INT movlw   SP_DF_TP_H      ; Compare High
        subwf   THERMO1_HIGH,w  ; subtract Y-X MESURE-CONSIGNE (High)

        btfss   STATUS,Z        ; Skip if egual
        goto    TEMP1_result    ; Saut au resultat
        movlw   SP_DF_TP_L      ; HIGH egal alors compar LOW
        subwf   THERMO1_LOW,w   ; subtract Y-X MESURE-CONSIGNE (Low)
TEMP1_result
        btfss   STATUS,Z        ; Skip if egual (X=Y alors Z=1)
        GOTO    TEMP1_res2
        GOTO    TEMP1_OFF       ; = alors suite
TEMP1_res2
        btfsc   STATUS,C        ; Skip if egual (X<Y alors C=0)
        GOTO    TEMP1_res3
        GOTO    TEMP1_OFF       ; < alors traitement Mesure<consigne
TEMP1_res3
        btfss   STATUS,C        ; Skip if egual (X>Y alors C=1)
        GOTO    TEMP1_suite
        GOTO    TEMP1_ON        ; > alors traitement Mesure>consigne
TEMP1_OFF
        bcf     SEUIL_TEMP1
        bra     TEMP1_suite
TEMP1_ON
        bsf     SEUIL_TEMP1

TEMP1_suite
;---------------------------- DEFAUTS ------------------------------ TEMP1 -----
        btfsc   FLAG_REBOND_2         ; Test si pas Seuil TEMP
        bsf     DEF_TEMP1           ; SET Memo Defaut TEMP
        btfsc   BP_ONOFF            ; RESET Defaut?  (Appui sur BP ON/OFF)
        bcf     DEF_TEMP1           ; RESET Memo Defaut TEMP

;------------------------------ SEUIL ------------------------ TEMPERATURE 2 ---

        movlw   SP_DF_TP_H          ; Compare High
        subwf   THERMO2_HIGH,w      ; subtract Y-X MESURE-CONSIGNE (High)

        btfss   STATUS,Z            ; Skip if egual
        goto    TEMP2_result        ; Saut au resultat
        movlw   SP_DF_TP_L          ; HIGH egal alors compar LOW
        subwf   THERMO2_LOW,w       ; subtract Y-X MESURE-CONSIGNE (Low)
TEMP2_result
        btfss   STATUS,Z            ; Skip if egual (X=Y alors Z=1)
        GOTO    TEMP2_res2
        GOTO    TEMP2_OFF           ; = alors suite
TEMP2_res2
        btfsc   STATUS,C            ; Skip if egual (X<Y alors C=0)
        GOTO    TEMP2_res3
        GOTO    TEMP2_OFF         ; < alors traitement Mesure<consigne
TEMP2_res3
        btfss   STATUS,C            ; Skip if egual (X>Y alors C=1)
        GOTO    TEMP2_suite
        GOTO    TEMP2_ON           ; > alors traitement Mesure>consigne
TEMP2_OFF
        bcf     SEUIL_TEMP2
        bra     TEMP2_suite
TEMP2_ON
        bsf     SEUIL_TEMP2

TEMP2_suite
;---------------------------- DEFAUTS ------------------------------ TEMP2 -----
        btfsc   FLAG_REBOND_3       ; Test si pas Seuil TEMP
        bsf     DEF_TEMP2           ; SET Memo Defaut TEMP
        btfsc   BP_ONOFF            ; RESET Defaut?  (Appui sur BP ON/OFF)
        bcf     DEF_TEMP2           ; RESET Memo Defaut TEMP

;------------------------- BIT CLIGNOTANT --------------------------------------

        movlw   d'128'          ; W = Valeur
        cpfsgt  COMPT_ISR        ; Compteur > valeur (W)
        bsf     FLAG_Flash50    ; NON alors Set bit
        cpfslt  COMPT_ISR        ; Compteur < valeur (W)
        bcf     FLAG_Flash50    ; NON alors Reset bit

        movlw   d'08'           ; W = Valeur
        cpfsgt  COMPT_ISR        ; Compteur > valeur (W)
        bsf     FLAG_Flash10    ; NON alors Set bit
        cpfslt  COMPT_ISR        ; Compteur < valeur (W)
        bcf     FLAG_Flash10    ; NON alors Reset bit

;------------------------------ SEUIL ----------------------- PRECHAUFFAGE 1 ---

        movlw   SP_CHAUF_H      ; Compare High
        subwf   THERMO1_HIGH,w  ; subtract Y-X MESURE-CONSIGNE (High)

        btfss   STATUS,Z        ; Skip if egual
        goto    CHAUF1_result    ; Saut au resultat
        movlw   SP_CHAUF_L      ; HIGH egal alors compar LOW
        subwf   THERMO1_LOW,w   ; subtract Y-X MESURE-CONSIGNE (Low)
CHAUF1_result
        btfss   STATUS,Z        ; Skip if egual (X=Y alors Z=1)
        GOTO    CHAUF1_res2
        GOTO    CHAUF1_OFF       ; = alors suite
CHAUF1_res2
        btfsc   STATUS,C        ; Skip if egual (X<Y alors C=0)
        GOTO    CHAUF1_res3
        GOTO    CHAUF1_ON        ; < alors traitement Mesure<consigne
CHAUF1_res3
        btfss   STATUS,C        ; Skip if egual (X>Y alors C=1)
        GOTO    CHAUF1_suite
        GOTO    CHAUF1_OFF       ; > alors traitement Mesure>consigne
CHAUF1_OFF
        bcf     FLAG_Prechauf1
        bra     CHAUF1_suite
CHAUF1_ON
        ;bsf     FLAG_Prechauf1

CHAUF1_suite
;------------------------------ SEUIL ----------------------- PRECHAUFFAGE 2 ---

        movlw   SP_CHAUF_H      ; Compare High
        subwf   THERMO2_HIGH,w  ; subtract Y-X MESURE-CONSIGNE (High)

        btfss   STATUS,Z        ; Skip if egual
        goto    CHAUF2_result    ; Saut au resultat
        movlw   SP_CHAUF_L      ; HIGH egal alors compar LOW
        subwf   THERMO2_LOW,w   ; subtract Y-X MESURE-CONSIGNE (Low)
CHAUF2_result
        btfss   STATUS,Z        ; Skip if egual (X=Y alors Z=1)
        GOTO    CHAUF2_res2
        GOTO    CHAUF2_OFF       ; = alors suite
CHAUF2_res2
        btfsc   STATUS,C        ; Skip if egual (X<Y alors C=0)
        GOTO    CHAUF2_res3
        GOTO    CHAUF2_ON        ; < alors traitement Mesure<consigne
CHAUF2_res3
        btfss   STATUS,C        ; Skip if egual (X>Y alors C=1)
        GOTO    CHAUF2_suite
        GOTO    CHAUF2_OFF       ; > alors traitement Mesure>consigne
CHAUF2_OFF
        bcf     FLAG_Prechauf2
        bra     CHAUF2_suite
CHAUF2_ON
        ;bsf     FLAG_Prechauf2

CHAUF2_suite
;------------------------------- LED ------------------------------- CC1 -------
        btfss   DEF_CC1         ; Test si defaut CC 1
        bra     CC1_reset       ; SET LED CC 1
        btfsc   CT_CC1          ; Test si Contact CC 1
        bra     CC1_fixe         ; Alume LED Defaut
        bra     CC1_Cligotant
CC1_reset
        bcf    LED_CC1
        bra    CC1_exit
CC1_fixe
        bsf    LED_CC1
        bra    CC1_exit
CC1_Cligotant
        btfsc   FLAG_Flash50    ; Test Bit clignotant OFF
        bsf     LED_CC1         ; SET LED
        btfss   FLAG_Flash50    ; Test Bit clignotant ON
        bcf     LED_CC1         ; RESET LED
CC1_exit
;------------------------------- LED ------------------------------- CC2 -------
        btfss   DEF_CC2         ; Test si defaut CC 2
        bra     CC2_reset       ; SET LED CC 1
        btfsc   CT_CC2          ; Test si Contact CC 2
        bra     CC2_fixe         ; Alume LED Defaut
        bra     CC2_Cligotant
CC2_reset
        bcf    LED_CC2
        bra    CC2_exit
CC2_fixe
        bsf    LED_CC2
        bra    CC2_exit
CC2_Cligotant
        btfsc   FLAG_Flash50    ; Test Bit clignotant OFF
        bsf     LED_CC2         ; SET LED
        btfss   FLAG_Flash50    ; Test Bit clignotant ON
        bcf     LED_CC2         ; RESET LED
CC2_exit
;------------------------------- LED ------------------------------ INT1 -------
        btfss   DEF_INT1        ; Test si defaut INT 1
        bra     INT1_reset      ; SET LED CC 1
        btfsc   CT_CC1        ; Test si Contact INT 1
        bra     INT1_fixe       ; Alume LED Defaut
        bra     INT1_Cligotant
INT1_reset
        bcf    LED_INT1
        bra    INT1_exit
INT1_fixe
        bsf    LED_INT1
        bra    INT1_exit
INT1_Cligotant
        btfsc   FLAG_Flash50    ; Test Bit clignotant OFF
        bsf     LED_INT1        ; SET LED
        btfss   FLAG_Flash50    ; Test Bit clignotant ON
        bcf     LED_INT1        ; RESET LED
INT1_exit

;------------------------------- LED ------------------------------ INT2 -------
        btfss   DEF_INT2        ; Test si defaut INT 2
        bra     INT2_reset      ; SET LED CC 1
        btfsc   CT_CC2        ; Test si Contact INT 2
        bra     INT2_fixe       ; Alume LED Defaut
        bra     INT2_Cligotant
INT2_reset
        bcf     LED_INT2
        bra     INT2_exit
INT2_fixe
        bsf     LED_INT2
        bra     INT2_exit
INT2_Cligotant
        btfsc   FLAG_Flash50    ; Test Bit clignotant OFF
        bsf     LED_INT2        ; SET LED
        btfss   FLAG_Flash50    ; Test Bit clignotant ON
        bcf     LED_INT2        ; RESET LED
INT2_exit

;------------------------------- LED ------------------------------- TEMP1 -----
        btfss   DEF_TEMP1       ; Test si defaut TEMP1
        bra     TEMP1_reset     ; SET LED CC 1
        btfsc   SEUIL_TEMP1        ; Test si Contact TEMP1
        bra     TEMP1_fixe      ; Alume LED Defaut
        bra     TEMP1_Cligotant
TEMP1_reset
        bcf     LED_TEMP1
        bra     TEMP1_exit
TEMP1_fixe
        bsf     LED_TEMP1
        bra     TEMP1_exit
TEMP1_Cligotant
        btfsc   FLAG_Flash10     ; Test Bit clignotant OFF
        bsf     LED_TEMP1        ; SET LED
        btfss   FLAG_Flash10     ; Test Bit clignotant ON
        bcf     LED_TEMP1        ; RESET LED
TEMP1_exit

        btfss   DEF_TEMP2       ; Test si defaut TEMP2
        bra     TEMP2_reset     ; SET LED CC 1
        btfsc   SEUIL_TEMP2        ; Test si Contact TEMP2
        bra     TEMP2_fixe      ; Alume LED Defaut
        bra     TEMP2_Cligotant
TEMP2_reset
        bcf     LED_TEMP2
        bra     TEMP2_exit
TEMP2_fixe
        bsf     LED_TEMP2
        bra     TEMP2_exit
TEMP2_Cligotant
        btfsc   FLAG_Flash10    ; Test Bit clignotant OFF
        bsf     LED_TEMP2       ; SET LED
        btfss   FLAG_Flash10    ; Test Bit clignotant ON
        bcf     LED_TEMP2       ; RESET LED
TEMP2_exit

;------------------------------ RELAIS ---------------------------- MUTE 1 -----
        bcf     FLAG_MUTE1      ; Reset Flag
        btfsc   DEF_CC1         ; Defaut COURANT CONTINU
        bsf     FLAG_MUTE1      ; Set Flag
        btfsc   DEF_INT1        ; Defaut INTERN
        bsf     FLAG_MUTE1      ; Set Flag
        btfsc   DEF_TEMP1       ; Defaut TEMPERATURE
        bsf     FLAG_MUTE1      ; Set Flag

        btfsc   FLAG_MUTE1      ; Test Flag Mute
        bsf     MUTE1           ; UNMUTE
        btfss   FLAG_MUTE1      ; Test Flag Mute
        bsf     MUTE1           ; MUTE

;------------------------------ RELAIS ---------------------------- MUTE 2 -----
        bcf     FLAG_MUTE2      ; Reset Flag
        btfsc   DEF_CC2         ; Defaut COURANT CONTINU
        bsf     FLAG_MUTE2      ; Set Flag
        btfsc   DEF_INT2        ; Defaut INTERN
        bsf     FLAG_MUTE2      ; Set Flag
        btfsc   DEF_TEMP2       ; Defaut TEMPERATURE
        bsf     FLAG_MUTE2      ; Set Flag

        btfsc   FLAG_MUTE2      ; Test Flag Mute
        bsf     MUTE2           ; UNMUTE
        btfss   FLAG_MUTE2      ; Test Flag Mute
        bsf     MUTE2           ; MUTE

;----------------------- DETECTION PHASE ARRET ----------------- BP ON/OFF -----

                btfss   BP_ONOFF            ; Test si appuie BP
                bra     ARRET_OFF_BP
ARRET_ON_BP     incf    COMPT_BP_L ,F       ; Add one to LSB
                btfsc   STATUS,Z            ; Overflowed?
                incf    COMPT_BP_H,F        ; Yes, add one to MSB
                bra     TEST_ARRET_BP
ARRET_OFF_BP    clrf    COMPT_BP_L          ; RAZ compteur
                clrf    COMPT_BP_H          ; RAZ compteur
TEST_ARRET_BP   movlw   0x40                ; Valeur Compteur
                cpfsgt  COMPT_BP_H          ; Compteur > Valeur
                bra     TEST_ARRET_BP_suite
                bra     PHASE_ARRET         ; Saute phase d'arret
TEST_ARRET_BP_suite

;----------------------- DETECTION PHASE ARRET ----------------- CT EXTERNE ----

                btfss   FLAG_FD_1           ; Test Front Descendant Contact REM
                bra     TEST_ARRET_CT_suite ; Non alors continu
                bra     PHASE_ARRET         ; Oui alors phase d'arret

TEST_ARRET_CT_suite

;*******************************************************************************
;*********************** BOUCLE SECONDAIRE  *************** MEDIUM PRIORITY ****
;*******************************************************************************

                btfss   cycleL,0
                bra     Low_Priority
                btfss   cycleL,1
                bra     Low_Priority
                btfss   cycleL,2
                bra     Low_Priority
                btfss   cycleL,3
                bra     Low_Priority
                btfss   cycleL,4
                bra     Low_Priority

;--------------------------- PWM LED--------------------------------------------
                bcf     FLAG_FlashON    ; RESET flag cligno LED BLEU
                btfsc   FLAG_Prechauf1  ; Test si Prechauffage 1
                bsf     FLAG_FlashON    ; Oui alos SET flag cligno LED BLEU
                btfsc   FLAG_Prechauf2  ; Test si Prechauffage 2
                bsf     FLAG_FlashON    ; Oui alos SET flag cligno LED BLEU

                btfss   FLAG_FlashON    ; Mode clignotement ?
                bra     PWMLED_Max      ; Non alors junp to LED MAX
                bra     PWMLED_flash    ; Oui alors junp to Flash

PWMLED_flash    movlw   0x70            ; PWM_LED = 01
                cpfseq  PWM_LED
                bra     PWMLED_suite
                bsf     FLAG_Invers     ; PWM_LED flag pour Incrementation

PWMLED_suite    movlw   0xFE            ; PWM_LED = 01
                cpfseq  PWM_LED
                bra     PWMLED_suite2
                bcf     FLAG_Invers     ; PWM_LED flag pour decrementation

PWMLED_suite2   btfsc   FLAG_Invers     ; Sens Decrementation?
                incf    PWM_LED,f       ; Incremente intensit LED
                btfss   FLAG_Invers         ; Sens Decrementation?
                decf    PWM_LED,f       ; Incremente intensit LED
                bra     PWMLED_exit

PWMLED_Max      movff   PWM_LED_MAX,PWM_LED         ; intensit LED  100%

PWMLED_exit

;*******************************************************************************
;*********************** BOUCLE TERCIAIRE  ******************* LOW PRIORITY ****
;*******************************************************************************

Low_Priority

        movlw   d'254'           ; Valeur Cycle pour LOW PRIORITY

        cpfsgt  cycleH          ; Compare si cycle > valeur (W)
        bra     fin             ; Sinon suite
        clrf    cycleL          ; RESET Cycle
        clrf    cycleH          ; RESET Cycle

;--------------- REGUL PID 1 ------------------- COMPARAISON CONSIGNE/MESURE ---

        movlw   SP_REGUL_H     ; Compare High
        subwf   THERMO1_HIGH,w  ; subtract Y-X MESURE-CONSIGNE (High)

        btfss   STATUS,Z        ; Skip if egual
        goto    comp1_res        ; Saut au resultat
        movlw   SP_REGUL_L      ; HIGH egal alors compar LOW
        subwf   THERMO1_LOW,w   ; subtract Y-X MESURE-CONSIGNE (Low)

comp1_res
        btfss   STATUS,Z        ; Skip if egual (X=Y alors Z=1)
        GOTO    comp1_res2
        GOTO    exit1            ; = alors suite
comp1_res2
        btfsc   STATUS,C        ; Skip if egual (X<Y alors C=0)
        GOTO    comp1_res3
        GOTO    mesure1_inf      ; < alors traitement Mesure<consigne
comp1_res3
        btfss   STATUS,C        ; Skip if egual (X>Y alors C=1)
        GOTO    exit1
        GOTO    mesure1_sup      ; > alors traitement Mesure>consigne

;--------------- REGUL PID 1 ---------------------------- MESURE < CONSIGNE ----
mesure1_inf
        movf    PWM1_MIN_HIGH,w  ; Compare High
        subwf   PWM1_HIGH,w      ; subtract Y-X PWM-PWM_MIN High

        btfss   STATUS,Z        ; Skip if egual
        goto    mesure1_res_inf  ; Saut au resultat
        movf    PWM1_MIN_LOW,w   ; HIGH egal alors compar LOW
        subwf   PWM1_LOW,w       ; subtract Y-X PWM-PWM_MIN Low

mesure1_res_inf
        btfss   STATUS,Z        ; Skip if egual (X=Y alors Z=1)
        GOTO    mesure1_res_inf2
        GOTO    exit1            ; = alors suite
mesure1_res_inf2
        btfss   STATUS,C        ; Skip if superior (X>Y alors C=1)
        GOTO    exit1
        movf    PWM1_LOW,W       ; Does LSB contain zero?
        btfsc   STATUS,Z        ; Skip if =0
        decf    PWM1_HIGH,F      ; Not =0 then decrement the MSB
        decf    PWM1_LOW,F       ; decrement the LSB
        GOTO    exit1
;--------------- REGUL PID 1 ---------------------------- MESURE > CONSIGNE ----
mesure1_sup
        movf    PWM1_MAX_HIGH,w  ; Compare High
        subwf   PWM1_HIGH,w      ; subtract Y-X MIN-PWM (High)

        btfss   STATUS,Z        ; Skip if egual
        goto    mesure1_res_sup
        movf    PWM1_MAX_LOW,w   ; yes, they are equal then compare low
        subwf   PWM1_LOW,w       ; subtract Y-X MIN-PWM-CONSIGNE Low

mesure1_res_sup
        btfsc   STATUS,C        ; Skip if inferior (X<Y alors C=0)
        GOTO    exit1
        incf    PWM1_LOW,F       ; Add one to LSB
        btfsc   STATUS,Z        ; Overflowed?
        incf    PWM1_HIGH,F      ; Yes, add one to MSB
        GOTO    exit1
exit1

;--------------- REGUL PID 2 ------------------- COMPARAISON CONSIGNE/MESURE ---

        movlw   SP_REGUL_H     ; Compare High
        subwf   THERMO2_HIGH,w  ; subtract Y-X MESURE-CONSIGNE (High)

        btfss   STATUS,Z        ; Skip if egual
        goto    comp2_res        ; Saut au resultat
        movlw   SP_REGUL_L      ; HIGH egal alors compar LOW
        subwf   THERMO2_LOW,w   ; subtract Y-X MESURE-CONSIGNE (Low)

comp2_res
        btfss   STATUS,Z        ; Skip if egual (X=Y alors Z=1)
        GOTO    comp2_res2
        GOTO    exit2            ; = alors suite
comp2_res2
        btfsc   STATUS,C        ; Skip if egual (X<Y alors C=0)
        GOTO    comp2_res3
        GOTO    mesure2_inf      ; < alors traitement Mesure<consigne
comp2_res3
        btfss   STATUS,C        ; Skip if egual (X>Y alors C=1)
        GOTO    exit2
        GOTO    mesure2_sup      ; > alors traitement Mesure>consigne

;--------------- REGUL PID 2 ---------------------------- MESURE < CONSIGNE ----
mesure2_inf
        movf    PWM2_MIN_HIGH,w  ; Compare High
        subwf   PWM2_HIGH,w      ; subtract Y-X PWM-PWM_MIN High

        btfss   STATUS,Z        ; Skip if egual
        goto    mesure2_res_inf  ; Saut au resultat
        movf    PWM2_MIN_LOW,w   ; HIGH egal alors compar LOW
        subwf   PWM2_LOW,w       ; subtract Y-X PWM-PWM_MIN Low

mesure2_res_inf
        btfss   STATUS,Z        ; Skip if egual (X=Y alors Z=1)
        GOTO    mesure2_res_inf2
        GOTO    exit2            ; = alors suite
mesure2_res_inf2
        btfss   STATUS,C        ; Skip if superior (X>Y alors C=1)
        GOTO    exit2
        movf    PWM2_LOW,W       ; Does LSB contain zero?
        btfsc   STATUS,Z        ; Skip if =0
        decf    PWM2_HIGH,F      ; Not =0 then decrement the MSB
        decf    PWM2_LOW,F       ; decrement the LSB
        GOTO    exit2
;--------------- REGUL PID 1 ---------------------------- MESURE > CONSIGNE ----
mesure2_sup
        movf    PWM2_MAX_HIGH,w  ; Compare High
        subwf   PWM2_HIGH,w      ; subtract Y-X MIN-PWM (High)

        btfss   STATUS,Z        ; Skip if egual
        goto    mesure2_res_sup
        movf    PWM2_MAX_LOW,w   ; yes, they are equal then compare low
        subwf   PWM2_LOW,w       ; subtract Y-X MIN-PWM-CONSIGNE Low

mesure2_res_sup
        btfsc   STATUS,C        ; Skip if inferior (X<Y alors C=0)
        GOTO    exit2
        incf    PWM2_LOW,F       ; Add one to LSB
        btfsc   STATUS,Z        ; Overflowed?
        incf    PWM2_HIGH,F      ; Yes, add one to MSB
        GOTO    exit2
exit2

;-------------------------- PRECHAUFFAGE 1 ? -----------------------------------
        btfss   FLAG_Prechauf1  ; Prchauffage 1?
        bra     PWM1_START
        bra     PWM1_STOP
PWM1_STOP
        MOVLW   0x00            ; PWM 1  0%
        MOVWF   CCPR2L          ; SET high 8 bits of PWM duty cycle
        bcf     CCP2CON,4       ; SET 2 bits of PWM Duty cycle DC1B
        bcf     CCP2CON,5       ; SET 2 bits of PWM Duty cycle DC1B
        bra     PWM2_TEST
;----------------------------- PWM 1 --------------------------------- AUTO ----
PWM1_START
        movf    PWM1_LOW,W
        movwf   temp1
        rrncf   temp1,F
        rrncf   temp1,F

        btfsc   PWM1_HIGH,0
        bsf     temp1,6         ; SET bits 6 of PWM_LOW
        btfss   PWM1_HIGH,0
        bcf     temp1,6         ; RESET bits 6 of PWM_LOW

        btfsc   PWM1_HIGH,1
        bsf     temp1,7         ; SET bits 7 of PWM_LOW
        btfss   PWM1_HIGH,1
        bcf     temp1,7         ; RESET bits 7 of PWM_LOW

        movf    temp1,w
        movwf   CCPR2L
        btfsc   PWM1_LOW,0
        bsf     CCP2CON,4       ; SET bits 0 of PWM Duty cycle DC1B4
        btfss   PWM1_LOW,0
        bcf     CCP2CON,4       ; RESET bits 0 of PWM Duty cycle DC1B4

        btfsc   PWM1_LOW,1
        bsf     CCP2CON,5       ; SET bits 1 of PWM Duty cycle DC1B5
        btfss   PWM1_LOW,1
        bcf     CCP2CON,5       ; RESET bits 1 of PWM Duty cycle DC1B5
;-------------------------- PRECHAUFFAGE 2 ? -----------------------------------
PWM2_TEST
        btfss   FLAG_Prechauf2  ; Prchauffage 2?
        bra     PWM2_START
        bra     PWM2_STOP
PWM2_STOP
        MOVLW   0x00            ; PWM 2  0%
        MOVWF   CCPR1L          ; SET high 8 bits of PWM duty cycle
        bcf     CCP1CON,4       ; SET 2 bits of PWM Duty cycle DC1B
        bcf     CCP1CON,5       ; SET 2 bits of PWM Duty cycle DC1B
        bra     fin
;----------------------------- PWM 2 --------------------------------- AUTO ----
PWM2_START
        movf    PWM2_LOW,W
        movwf   temp1
        rrncf   temp1,F
        rrncf   temp1,F

        btfsc   PWM2_HIGH,0
        bsf     temp1,6         ; SET bits 6 of PWM_LOW
        btfss   PWM2_HIGH,0
        bcf     temp1,6         ; RESET bits 6 of PWM_LOW

        btfsc   PWM2_HIGH,1
        bsf     temp1,7         ; SET bits 7 of PWM_LOW
        btfss   PWM2_HIGH,1
        bcf     temp1,7         ; RESET bits 7 of PWM_LOW

        movf    temp1,w
        movwf   CCPR1L


        btfsc   PWM2_LOW,0
        bsf     CCP1CON,4       ; SET bits 0 of PWM Duty cycle DC1B4
        btfss   PWM2_LOW,0
        bcf     CCP1CON,4       ; RESET bits 0 of PWM Duty cycle DC1B4

        btfsc   PWM2_LOW,1
        bsf     CCP1CON,5       ; SET bits 1 of PWM Duty cycle DC1B5
        btfss   PWM2_LOW,1
        bcf     CCP1CON,5       ; RESET bits 1 of PWM Duty cycle DC1B5

;************************** END OF PROGRAM *****************************
fin             goto    High_Priority
                END